@@ -1,25 +1,43 @@
-num2pal <- function(x, n = 5,
-					   style = "pretty",
-                       breaks = NULL,
-					   interval.closure="left",
-					   palette = NULL,
-					   auto.palette.mapping = TRUE,
-					   contrast = 1,
-					   legend.labels = NULL,
-					   colorNA = "#FF1414",
-					   legend.NA.text = "Missing",
-					   showNA=NA,
-					   process.colors=NULL,
-					   legend.format=list(scientific=FALSE),
+num2pal <- function(x, 
+					var,
+					call,
+					n = 5,
+					style = "pretty",
+					breaks = NULL,
+					interval.closure="left",
+					palette = NULL,
+					midpoint = NA, #auto.palette.mapping = TRUE,
+					contrast = 1,
+					legend.labels = NULL,
+					colorNA = "#FF1414",
+					colorNULL = "#FFFFFF",
+					legend.NA.text = "Missing",
+					showNA=NA,
+					process.colors=NULL,
+					legend.format=list(scientific=FALSE),
 					reverse=FALSE
 					) {
+	sel <- attr(x, "sel")
+	if (is.null(sel)) sel <- rep(TRUE, length(x))
+	
+	x[!sel] <- NA
+	
+	show.messages <- get(".tmapOptions", envir = .TMAP_CACHE)$show.messages
+	
+	
 	breaks.specified <- !is.null(breaks)
 	is.cont <- (style=="cont" || style=="order")
 	
 	if (is.cont) {
 		style <- ifelse(style=="order", "quantile", "fixed")
 
+		
+		
 		if (style=="fixed") {
+			if ((!is.null(breaks) || !is.null(legend.labels)) && ("n" %in% call)) {
+				warning("n will not be used since breaks and/or labels are specified. Therefore, n will be set to the number of breaks/labels.", call. = FALSE)
+			}
+
 			custom_breaks <- breaks
 			
 			if (!is.null(custom_breaks)) {
@@ -28,27 +46,35 @@ num2pal <- function(x, n = 5,
 				breaks <- range(x, na.rm = TRUE)
 			}
 			breaks <- cont_breaks(breaks, n=101)
+		} else {
+			if ("breaks" %in% call) warning("breaks cannot be set for style = \"order\".", 
+											ifelse("labels" %in% call, "", " Breaks labels can be set with the argument labels."), ifelse(any(c("labels", "n") %in% call), "", " The number of breaks can be specified with the argument n."),  call. = FALSE)
+			custom_breaks <- breaks
 		}
 		
 		if (is.null(legend.labels)) {
 			ncont <- n
 		} else {
 			if (!is.null(custom_breaks) && length(legend.labels) != n+1) {
-				warning("legend.labels not the same length as breaks", call.=FALSE)
+				warning("The length of legend.labels is ", length(legend.labels), ", which differs from the length of the breaks (", (n+1), "). Therefore, legend.labels will be ignored", call.=FALSE)
 				legend.labels <- NULL
 			} else {
+				if (style == "quantile" && ("n" %in% call)) {
+					warning("n will not be used since labels are specified. Therefore, n will be set to the number of labels.", call. = FALSE)
+				}
 				ncont <- length(legend.labels)	
 			}
 		}
-		
-		q <- num2breaks(x=x, n=101, style=style, breaks=breaks, approx=TRUE, interval.closure=interval.closure)
-		n <- length(q$brks) - 1
+		q <- num2breaks(x=x, n=101, style=style, breaks=breaks, approx=TRUE, interval.closure=interval.closure, var=var)
 	} else {
-		q <- num2breaks(x=x, n=n, style=style, breaks=breaks, interval.closure=interval.closure)
+		q <- num2breaks(x=x, n=n, style=style, breaks=breaks, interval.closure=interval.closure, var=var)
 	}
-
+	
+	
 	breaks <- q$brks
 	nbrks <- length(breaks)
+	n <- nbrks - 1
+	
 	int.closure <- attr(q, "intervalClosure")
 	
 	# reverse palette
@@ -59,86 +85,77 @@ num2pal <- function(x, n = 5,
 	
 	
 	# map palette
-	is.brewer <- palette[1] %in% rownames(brewer.pal.info)
+	is.palette.name <- palette[1] %in% rownames(tmap.pal.info)
 	
-	if (is.brewer) {
-		mc <- brewer.pal.info[palette, "maxcolors"]
-		pal.div <- (brewer.pal.info[palette, "category"]=="div")
+	is.brewer <- is.palette.name && tmap.pal.info[palette[1], "origin"] == "brewer"
+	is.viridis <- is.palette.name &&  tmap.pal.info[palette[1], "origin"] == "viridis"
+	
+	if (is.palette.name) {
+		mc <- tmap.pal.info[palette, "maxcolors"]
+		pal.div <- (!is.null(midpoint)) || (tmap.pal.info[palette, "category"]=="div")
 	} else {
-		palette.type <- palette_type(palette)
-		
-		if (auto.palette.mapping && palette.type=="cat") {
-			warning("could not determine whether palette is sequential or diverging. auto.palette.mapping will be set to FALSE.", call. = FALSE)
-			auto.palette.mapping <- FALSE
-		}
+		palette.type <- if (!is.null(midpoint)) "div" else palette_type(palette)
 		pal.div <- palette.type=="div"
-			
-		
-		colpal_light <- get_light(palette[c(1, length(palette)/2, length(palette))])
-		# figure out whether palette is diverging
-		pal.div <- ((colpal_light[2]>colpal_light[1] && colpal_light[2]>colpal_light[3]) || (colpal_light[2]<colpal_light[1] && colpal_light[2]<colpal_light[3]))
+	}
+	
+	if ((is.null(midpoint) || is.na(midpoint)) && pal.div) {
+		rng <- range(x, na.rm = TRUE)
+		if (rng[1] < 0 && rng[2] > 0 && is.null(midpoint)) {
+			if (show.messages) message("Variable \"", var, "\" contains positive and negative values, so midpoint is set to 0. Set midpoint = NA to show the full spectrum of the color palette.")
+			midpoint <- 0
+		} else {
+			if ((n %% 2) == 1) {
+				# number of classes is odd, so take middle class (average of those breaks)
+				midpoint <- mean(breaks[c((n+1) / 2, (n+3) / 2)])
+			} else {
+				midpoint <- breaks[(n+2) / 2]
+			}
+		}
 	}
 
-	if (auto.palette.mapping) {
-		if (is.brewer) {
-			colpal <- colorRampPalette(revPal(brewer.pal(mc, palette)), space="rgb")(101)
-		} else {
-			colpal <- colorRampPalette(revPal(palette), space="rgb")(101)
-		}
-		
-		ids <- if (pal.div) {
-			if (is.na(contrast[1])) contrast <- if (is.brewer) default_contrast_div(n) else c(0, 1)
-			map2divscaleID(breaks, n=101, contrast=contrast)
-		} else {
-			if (is.na(contrast[1])) contrast <- if (is.brewer) default_contrast_seq(n) else c(0, 1)
-			map2seqscaleID(breaks, n=101, contrast=contrast, breaks.specified=breaks.specified)
-		}
-		
-		legend.palette <- colpal[ids]
-		if (any(ids<51) && any(ids>51)) {
-			ids.neutral <- min(ids[ids>=51]-51) + 51
-			legend.neutral.col <- colpal[ids.neutral]
-		} else {
-			legend.neutral.col <- colpal[ids[round(((length(ids)-1)/2)+1)]]
-		}
-		
+	if (is.brewer) {
+		colpal <- colorRampPalette(revPal(brewer.pal(mc, palette)), space="rgb")(101)
+	} else if (is.viridis) {
+		colpal <- revPal(viridis(101, option = palette))
 	} else {
-		if (is.brewer) {
-			if (nbrks-1 > mc) {
-				legend.palette <- colorRampPalette(revPal(brewer.pal(mc, palette)), space="rgb")(nbrks-1)
-			} else legend.palette <- revPal(brewer.pal(nbrks-1, palette))
-		} else {
-			legend.palette <- colorRampPalette(revPal(palette), space="rgb")(nbrks-1) #rep(palette, length.out=nbrks-1)
-		}
-		neutralID <- if (pal.div) round(((length(legend.palette)-1)/2)+1) else 1
-		legend.neutral.col <- legend.palette[neutralID]
+		colpal <- colorRampPalette(revPal(palette), space="rgb")(101)
 	}
+	
+	ids <- if (pal.div) {
+		if (is.na(contrast[1])) contrast <- if (is.brewer) default_contrast_div(n) else c(0, 1)
+		map2divscaleID(breaks - midpoint, n=101, contrast=contrast)
+	} else {
+		if (is.na(contrast[1])) contrast <- if (is.brewer) default_contrast_seq(n) else c(0, 1)
+		map2seqscaleID(breaks, n=101, contrast=contrast, breaks.specified=breaks.specified)
+	}
+	
+	legend.palette <- colpal[ids]
+	if (any(ids<51) && any(ids>51)) {
+		ids.neutral <- min(ids[ids>=51]-51) + 51
+		legend.neutral.col <- colpal[ids.neutral]
+	} else {
+		legend.neutral.col <- colpal[ids[round(((length(ids)-1)/2)+1)]]
+	}
+		
 
 	legend.palette <- do.call("process_color", c(list(col=legend.palette), process.colors))
 	legend.neutral.col <- do.call("process_color", c(list(col=legend.neutral.col), process.colors))
 	colorNA <- do.call("process_color", c(list(col=colorNA), process.colors))
 	
-# 	if (!is.null(process.colors)) {
-# 		legend.palette <- process.colors(legend.palette)
-# 		legend.neutral.col <- process.colors(legend.neutral.col)
-# 		colorNA <- process.colors(colorNA)
-# 	}
-# 	
-# 	legend.palette <- get_alpha_col(legend.palette, alpha)
-# 	legend.neutral.col <- get_alpha_col(legend.neutral.col, alpha)
-# 	colorNA <- get_alpha_col(colorNA, alpha)
+
 	
 	
 	ids <- findCols(q)
 	cols <- legend.palette[ids]
-	anyNA <- any(is.na(cols))
+	anyNA <- any(is.na(cols) & sel)
 	breaks.palette <- legend.palette
 	if (anyNA) {
-		if (is.na(showNA)) showNA <- TRUE
+		if (is.na(showNA)) showNA <- any(is.na(cols) & sel)
 		cols[is.na(cols)] <- colorNA
 	} else {
 		if (is.na(showNA)) showNA <- FALSE
 	}
+	cols[!sel] <- colorNULL
 
 	if (is.cont) {
 		# recreate legend palette for continuous cases
