@@ -21,16 +21,16 @@ process_grid <- function(gt, bbx, proj, sasp) {
 		grid.custom.x <- !is.na(grid.x[1])
 		grid.custom.y <- !is.na(grid.y[1])
 		
-		if (!grid.custom.x) grid.x <- pretty(bbx[1,], n=grid.n.x)
-		if (!grid.custom.y) grid.y <- pretty(bbx[2,], n=grid.n.y)
+		if (!grid.custom.x) grid.x <- pretty(bbx[c(1,3)], n=grid.n.x)
+		if (!grid.custom.y) grid.y <- pretty(bbx[c(2,4)], n=grid.n.y)
 
 		## copy grid.x and y
 		grid.x.orig <- grid.x
 		grid.y.orig <- grid.y
 		
 		## crop
-		grid.x <- grid.x[grid.x>bbx[1,1] & grid.x<bbx[1,2]]
-		grid.y <- grid.y[grid.y>bbx[2,1] & grid.y<bbx[2,2]]
+		grid.x <- grid.x[grid.x>bbx[1] & grid.x<bbx[3]]
+		grid.y <- grid.y[grid.y>bbx[2] & grid.y<bbx[4]]
 		
 		## project grid lines
 		if (!is.na(grid.projection)) {
@@ -71,54 +71,97 @@ process_grid <- function(gt, bbx, proj, sasp) {
 			gnx2 <- gny2 <- NULL
 			
 			## determine limits
-			grid.x2.min <- min(min(grid.x2), bbx[1, 1], na.rm=TRUE)
-			grid.x2.max <- max(max(grid.x2), bbx[1, 2], na.rm=TRUE)
-			grid.y2.min <- min(min(grid.y2), bbx[2, 1], na.rm=TRUE)
-			grid.y2.max <- max(max(grid.y2), bbx[2, 2], na.rm=TRUE)
+			grid.x2.min <- min(min(grid.x2), bbx[1], na.rm=TRUE)
+			grid.x2.max <- max(max(grid.x2), bbx[3], na.rm=TRUE)
+			grid.y2.min <- min(min(grid.y2), bbx[2], na.rm=TRUE)
+			grid.y2.max <- max(max(grid.y2), bbx[4], na.rm=TRUE)
 			
 			# create SLDF with straight grid lines in grid lines projection
 			#bbx2 <- bb(bbx, ext=3)
+			
+
 			lnsList <- list(
-				if (is.na(grid.x2[1])) NULL else Lines(lapply(grid.x2, function(x) {
+				if (is.na(grid.x2[1])) NULL else st_multilinestring(lapply(grid.x2, function(x) {
 					m <- matrix(c(rep(x,100), seq(grid.y2.min, grid.y2.max, length.out=100)), ncol=2)
-					Line(m)
-				}), ID="x"),
-				if (is.na(grid.y2[1])) NULL else Lines(lapply(grid.y2, function(y) {
+				})),
+				if (is.na(grid.y2[1])) NULL else st_multilinestring(lapply(grid.y2, function(y) {
 					m <- matrix(c(seq(grid.x2.min, grid.x2.max, length.out=100), rep(y,100)), ncol=2)
-					Line(m)
-				}), ID="y")
+				}))
 			)
 			
-			lnsSel <- !sapply(lnsList, is.null)
+			
+			# lnsList <- list(
+			# 	if (is.na(grid.x2[1])) NULL else Lines(lapply(grid.x2, function(x) {
+			# 		m <- matrix(c(rep(x,100), seq(grid.y2.min, grid.y2.max, length.out=100)), ncol=2)
+			# 		Line(m)
+			# 	}), ID="x"),
+			# 	if (is.na(grid.y2[1])) NULL else Lines(lapply(grid.y2, function(y) {
+			# 		m <- matrix(c(seq(grid.x2.min, grid.x2.max, length.out=100), rep(y,100)), ncol=2)
+			# 		Line(m)
+			# 	}), ID="y")
+			# )
+			
+			lnsSel <- !vapply(lnsList, is.null, logical(1))
 			if (!any(lnsSel)) {
 				grid.co.x.lns <- numeric(0)
 				grid.co.y.lns <- numeric(0)
 			} else {
-				lns <- SpatialLinesDataFrame(SpatialLines(lnsList[lnsSel], proj4string = get_proj4(grid.projection, as.CRS = TRUE)), data.frame(ID=c("x", "y")[lnsSel]), match.ID=FALSE)
+				
+				lns <- st_sf(ID=c("x", "y")[lnsSel], geometry = st_sfc(lnsList[lnsSel], crs = grid.projection))
+				
+				#lns <- SpatialLinesDataFrame(SpatialLines(lnsList[lnsSel], proj4string = get_proj4(grid.projection, output="CRS")), data.frame(ID=c("x", "y")[lnsSel]), match.ID=FALSE)
 				
 				# project it to current projection
 				lns_proj <- set_projection(lns, projection = proj)
+
 				
 				# extract and normalize coordinates
-				grid.co.x.lns <- if (lnsSel[1]) lapply(lns_proj@lines[[1]]@Lines, function(l) {
-					lco <- attr(l, "coords")
-					lco[, 1] <- (lco[, 1]-bbx_orig[1,1]) / (bbx_orig[1,2] - bbx_orig[1,1])
-					lco[, 2] <- (lco[, 2]-bbx_orig[2,1]) / (bbx_orig[2,2] - bbx_orig[2,1])
-					lco
-				}) else numeric(0)
-				grid.co.y.lns <- if (lnsSel[2]) lapply(lns_proj@lines[[sum(lnsSel)]]@Lines, function(l) {
-					lco <- attr(l, "coords")
-					lco[, 1] <- (lco[, 1]-bbx_orig[1,1]) / (bbx_orig[1,2] - bbx_orig[1,1])
-					lco[, 2] <- (lco[, 2]-bbx_orig[2,1]) / (bbx_orig[2,2] - bbx_orig[2,1])
-					lco
-				}) else numeric(0)
+				
+				if (lnsSel[1]) {
+					xco <- st_coordinates(st_geometry(lns_proj)[1])
+					grid.co.x.lns <- lapply(1L:length(st_geometry(lns_proj)[1][[1]]), function(i) {
+						lco <- xco[xco[,3]==i, 1:2]
+						lco[, 1] <- (lco[, 1]-bbx_orig[1]) / (bbx_orig[3] - bbx_orig[1])
+						lco[, 2] <- (lco[, 2]-bbx_orig[2]) / (bbx_orig[4] - bbx_orig[2])
+						lco
+					})
+					xco <- NULL
+				} else {
+					grid.co.x.lns <- numeric(0)
+				}
+
+				if (lnsSel[2]) {
+					yco <- st_coordinates(st_geometry(lns_proj)[sum(lnsSel)])
+					grid.co.y.lns <- lapply(1L:length(st_geometry(lns_proj)[sum(lnsSel)][[1]]), function(i) {
+						lco <- yco[yco[,3]==i, 1:2]
+						lco[, 1] <- (lco[, 1]-bbx_orig[1]) / (bbx_orig[3] - bbx_orig[1])
+						lco[, 2] <- (lco[, 2]-bbx_orig[2]) / (bbx_orig[4] - bbx_orig[2])
+						lco
+					})
+					yco <- NULL
+				} else {
+					grid.co.x.lns <- numeric(0)
+				}
+				
+				# grid.co.x.lns <- if (lnsSel[1]) lapply(lns_proj@lines[[1]]@Lines, function(l) {
+				# 	lco <- attr(l, "coords")
+				# 	lco[, 1] <- (lco[, 1]-bbx_orig[1,1]) / (bbx_orig[1,2] - bbx_orig[1,1])
+				# 	lco[, 2] <- (lco[, 2]-bbx_orig[2,1]) / (bbx_orig[2,2] - bbx_orig[2,1])
+				# 	lco
+				# }) else numeric(0)
+				# grid.co.y.lns <- if (lnsSel[2]) lapply(lns_proj@lines[[sum(lnsSel)]]@Lines, function(l) {
+				# 	lco <- attr(l, "coords")
+				# 	lco[, 1] <- (lco[, 1]-bbx_orig[1,1]) / (bbx_orig[1,2] - bbx_orig[1,1])
+				# 	lco[, 2] <- (lco[, 2]-bbx_orig[2,1]) / (bbx_orig[2,2] - bbx_orig[2,1])
+				# 	lco
+				# }) else numeric(0)
 			}
 			lns <- NULL
 			lns_proj <- NULL
 		} else {
 			# normalize coordinates
-			grid.co.x <- (grid.x-bbx[1,1]) / (bbx[1,2] - bbx[1,1])
-			grid.co.y <- (grid.y-bbx[2,1]) / (bbx[2,2] - bbx[2,1])
+			grid.co.x <- (grid.x-bbx[1]) / (bbx[3] - bbx[1])
+			grid.co.y <- (grid.y-bbx[2]) / (bbx[4] - bbx[2])
 			
 		}
 		
@@ -234,32 +277,60 @@ plot_grid <- function(gt, scale, add.labels) {
 	# crop projected grid lines, and extract polylineGrob ingredients
 	if (!is.na(gt$grid.projection)) {
 		lnsList <- list(
-			if (any(selx)) Lines(lapply(gt$grid.co.x.lns, function(m) {
-				Line(m)
-			}), ID="x") else NULL,
-			if (any(sely)) Lines(lapply(gt$grid.co.y.lns, function(m) {
-				Line(m)
-			}), ID="y") else NULL
+			if (any(selx)) st_multilinestring(gt$grid.co.x.lns) else NULL,
+			if (any(sely)) st_multilinestring(gt$grid.co.y.lns) else NULL
 		)
-		lnsSel <- !sapply(lnsList, is.null)
+		
+		# lnsList <- list(
+		# 	if (any(selx)) Lines(lapply(gt$grid.co.x.lns, function(m) {
+		# 		Line(m)
+		# 	}), ID="x") else NULL,
+		# 	if (any(sely)) Lines(lapply(gt$grid.co.y.lns, function(m) {
+		# 		Line(m)
+		# 	}), ID="y") else NULL
+		# )
+		lnsSel <- !vapply(lnsList, is.null, logical(1))
 		if (!any(lnsSel)) {
 			grid.co.x.lns <- numeric(0)
 			grid.co.y.lns <- numeric(0)
 		} else {
-			lns <- SpatialLinesDataFrame(SpatialLines(lnsList[lnsSel]), data.frame(ID=c("x", "y")[lnsSel]), match.ID=FALSE)
-			lns_crop <- raster::crop(lns, bb(c(labelsYw + spacerY + marginY, 1, labelsXw + spacerX + marginX, 1)))
 			
-			cogridxlns <- if (any(selx)) do.call("rbind", mapply(function(l, i) {
-				co <- as.data.frame(attr(l, "coords"))
-				co$ID <- i
-				co
-			}, lns_crop@lines[[1]]@Lines, 1:length(lns_crop@lines[[1]]@Lines), SIMPLIFY=FALSE)) else numeric(0)
+			lns <- st_sf(ID=c("x", "y")[lnsSel], geometry = st_sfc(lnsList[lnsSel], crs = 4326)) # trick for 0-1 coordinates
 			
-			cogridylns <- if (any(sely)) do.call("rbind", mapply(function(l, i) {
-				co <- as.data.frame(attr(l, "coords"))
-				co$ID <- i
-				co
-			}, lns_crop@lines[[sum(lnsSel)]]@Lines, 1:length(lns_crop@lines[[sum(lnsSel)]]@Lines), SIMPLIFY=FALSE)) else numeric(0)
+			
+			#lns <- SpatialLinesDataFrame(SpatialLines(lnsList[lnsSel]), data.frame(ID=c("x", "y")[lnsSel]), match.ID=FALSE)
+			
+			sf_bbox <- tmaptools::bb_poly(bb(c(labelsYw + spacerY + marginY, labelsXw + spacerX + marginX, 1, 1)), projection = 4326)
+			
+			lns_crop <- suppressWarnings(suppressMessages(st_intersection(lns, sf_bbox)))
+			#lns_crop <- raster::crop(lns, bb(c(labelsYw + spacerY + marginY, 1, labelsXw + spacerX + marginX, 1)))
+			
+			
+			
+			if (any(selx)) {
+				cogridxlns <- as.data.frame(st_coordinates(st_geometry(lns_crop)[1])[,1:3])
+				names(cogridxlns) <- c("x", "y", "ID")
+			} else {
+				cogridxlns <- numeric(0)
+			}
+			
+			if (any(sely)) {
+				cogridylns <- as.data.frame(st_coordinates(st_geometry(lns_crop)[sum(lnsSel)])[,1:3])
+				names(cogridylns) <- c("x", "y", "ID")
+			} else {
+				cogridylns <- numeric(0)
+			}
+			# cogridxlns <- if (any(selx)) do.call("rbind", mapply(function(l, i) {
+			# 	co <- as.data.frame(attr(l, "coords"))
+			# 	co$ID <- i
+			# 	co
+			# }, lns_crop@lines[[1]]@Lines, 1:length(lns_crop@lines[[1]]@Lines), SIMPLIFY=FALSE)) else numeric(0)
+			# 
+			# cogridylns <- if (any(sely)) do.call("rbind", mapply(function(l, i) {
+			# 	co <- as.data.frame(attr(l, "coords"))
+			# 	co$ID <- i
+			# 	co
+			# }, lns_crop@lines[[sum(lnsSel)]]@Lines, 1:length(lns_crop@lines[[sum(lnsSel)]]@Lines), SIMPLIFY=FALSE)) else numeric(0)
 			
 		}
 		
@@ -321,16 +392,29 @@ plot_grid <- function(gt, scale, add.labels) {
 get_gridline_labels <- function(lco, xax=NA, yax=NA) {
 	k <- length(lco)
 	d <- ifelse(!is.na(xax), 1, 2)
-	lns <- SpatialLines(mapply(function(m, id){
-		Lines(list(Line(m)), ID=id)
-	}, lco, 1:k, SIMPLIFY=FALSE))
+	
+	
+	lns <- st_sf(geometry=st_sfc(lapply(lco, function(l) {
+		st_linestring(l)
+	})), crs = 4326) # trick for 0-1 coordinates
+
+	# lns <- SpatialLines(mapply(function(m, id){
+	# 	Lines(list(Line(m)), ID=id)
+	# }, lco, 1:k, SIMPLIFY=FALSE))
+	
+	
+	
 	if (!is.na(xax)) {
-		ax <- SpatialLines(list(Lines(Line(matrix(c(0, 1, xax, xax), nrow=2)), ID="base")))
+		ax <- st_sf(geometry=st_sfc(st_linestring(matrix(c(0, 1, xax, xax), nrow=2))), crs = 4326)
+		#ax <- SpatialLines(list(Lines(Line(matrix(c(0, 1, xax, xax), nrow=2)), ID="base")))
 	} else {
-		ax <- SpatialLines(list(Lines(Line(matrix(c(yax, yax, 0, 1), nrow=2)), ID="base")))
+		ax <- st_sf(geometry=st_sfc(st_linestring(matrix(c(yax, yax, 0, 1), nrow=2))), crs = 4326)
+		#ax <- SpatialLines(list(Lines(Line(matrix(c(yax, yax, 0, 1), nrow=2)), ID="base")))
 	}
-	gint <- gIntersects(lns, ax, byid = TRUE)
-	ins <- sapply(lco, function(m) {
+	gint <- suppressMessages(st_intersects(lns, ax, sparse = FALSE, prepared = FALSE))[,1]
+	
+#	gint <- gIntersects(lns, ax, byid = TRUE)
+	ins <- vapply(lco, function(m) {
 		l <- m[1,]
 		if (!is.na(xax)) {
 			res <- l[1] >= 0 && l[1] <= 1 && l[2] >= xax && l[2] <= 1
@@ -338,31 +422,31 @@ get_gridline_labels <- function(lco, xax=NA, yax=NA) {
 			res <- l[1] >= yax && l[1] <= 1 && l[2] >= 0 && l[2] <= 1
 		}
 		if (res) l[d] else -1
-	})
+	}, numeric(1))
 	cogrid <- ifelse(gint, 0, ins)
 	if (any(gint)) {
-		gints <- gIntersection(lns[gint, ], ax, byid = TRUE)
-		cogrid[gint] <- gints@coords[,d]
+		gints <- suppressMessages(st_intersection(lns[gint, ], ax))
+		cogrid[gint] <- st_coordinates(gints)[,d]
 	}
 	cogrid
 }
 
 
-plot_symbols <- function(co.npc, g, gt, lineInch, i, k) {
-	symbolH <- convertHeight(unit(lineInch, "inch"), "npc", valueOnly=TRUE) * gt$scale
-	symbolW <- convertWidth(unit(lineInch, "inch"), "npc", valueOnly=TRUE) * gt$scale
+plot_symbols <- function(co.native, g, gt, lineInch, lineNatH, lineNatW, i, k) {
+	symbolH <- lineNatH * gt$scale
+	symbolW <- lineNatW * gt$scale
 	shapeLib <- get(".shapeLib", envir = .TMAP_CACHE)
 	justLib <- get(".justLib", envir = .TMAP_CACHE)
 
 	with(g, {
-		npol <- nrow(co.npc)
+		npol <- nrow(co.native)
 		if (length(symbol.size)!=npol) {
 			if (length(symbol.size)!=1) warning("less symbol size values than objects", call. = FALSE)
 			symbol.size <- rep(symbol.size, length.out=npol)
 		}
 
-		size.npc.w <- convertWidth(unit(symbol.size, "inch"), "npc", valueOnly = TRUE)
-		size.npc.h <- convertHeight(unit(symbol.size, "inch"), "npc", valueOnly = TRUE)
+		size.native.w <- convertWidth(unit(symbol.size, "inch"), "native", valueOnly = TRUE)
+		size.native.h <- convertHeight(unit(symbol.size, "inch"), "native", valueOnly = TRUE)
 
 		# determine justification per symbol
 		just <- g$symbol.misc$just
@@ -372,14 +456,14 @@ plot_symbols <- function(co.npc, g, gt, lineInch, i, k) {
 				if (is.na(js[1])) just else js
 			} else just
 		})
-		justs.x <- sapply(justs, "[[", 1)
-		justs.y <- sapply(justs, "[[", 2)
-		justx <- size.npc.w * (justs.x-.5)
-		justy <- size.npc.h * (justs.y-.5)
+		justs.x <- vapply(justs, "[[", numeric(1), 1)
+		justs.y <- vapply(justs, "[[", numeric(1), 2)
+		justx <- size.native.w * (justs.x-.5)
+		justy <- size.native.h * (justs.y-.5)
 		
 		# adjust the coordinates
-		co.npc[, 1] <- co.npc[, 1] + symbol.xmod * symbolW + justx * lineInch * 2 / 3
-		co.npc[, 2] <- co.npc[, 2] + symbol.ymod * symbolH + justy * lineInch * 2 / 3
+		co.native[, 1] <- co.native[, 1] + symbol.xmod * symbolW + justx * lineNatW * 2 / 3
+		co.native[, 2] <- co.native[, 2] + symbol.ymod * symbolH + justy * lineNatH * 2 / 3
 		
 		sel <- !is.na(symbol.size) & !is.na(symbol.col) & !is.na(symbol.shape)
 		
@@ -387,21 +471,22 @@ plot_symbols <- function(co.npc, g, gt, lineInch, i, k) {
 		if (!any(sel)) return(NULL)
 		
 		if (!all(sel)) {
-			co.npc <- co.npc[sel, , drop=FALSE]
+			co.native <- co.native[sel, , drop=FALSE]
 			symbol.size <- symbol.size[sel]
 			symbol.col <- symbol.col[sel]
 			symbol.shape <- symbol.shape[sel]
+			npol <- sum(sel)
 		}
 		symbol.size <- symbol.size * lineInch
 		
 		if (length(symbol.size)!=1) {
 			decreasing <- order(-symbol.size)
-			co.npc2 <- co.npc[decreasing,,drop=FALSE]
+			co.native2 <- co.native[decreasing,,drop=FALSE]
 			symbol.size2 <- symbol.size[decreasing]
 			symbol.shape2 <- symbol.shape[decreasing]
 			symbol.col2 <- symbol.col[decreasing]
 		} else {
-			co.npc2 <- co.npc
+			co.native2 <- co.native
 			symbol.size2 <- symbol.size
 			symbol.shape2 <- symbol.shape
 			symbol.col2 <- symbol.col
@@ -423,12 +508,12 @@ plot_symbols <- function(co.npc, g, gt, lineInch, i, k) {
 					} else {
 						gList(shapeLib[[symbol.shape2[i]-999]], rectGrob(gp=gpar(fill=NA, col=bordercol, lwd=symbol.border.lwd)))	
 					}
-					gTree(children=grbs, vp=viewport(x=unit(co.npc2[i,1], "npc"), 
-														  y=unit(co.npc2[i,2], "npc"),
+					gTree(children=grbs, vp=viewport(x=unit(co.native2[i,1], "native"), 
+														  y=unit(co.native2[i,2], "native"),
 														  width=unit(symbol.size2[i]*2/3, "inch"),
 														  height=unit(symbol.size2[i]*2/3, "inch")))
 				} else {
-					pointsGrob(x=unit(co.npc2[i,1], "npc"), y=unit(co.npc2[i,2], "npc"),
+					pointsGrob(x=unit(co.native2[i,1], "native"), y=unit(co.native2[i,2], "native"),
 							   size=unit(symbol.size2[i], "inch"),
 							   pch=symbol.shape2[i],
 							   gp=gpars[[i]])
@@ -436,7 +521,7 @@ plot_symbols <- function(co.npc, g, gt, lineInch, i, k) {
 			})
 			x <- gTree(children=do.call(gList, grobs), name=idName)
 		} else {
-			pointsGrob(x=unit(co.npc2[,1], "npc"), y=unit(co.npc2[,2], "npc"),
+			pointsGrob(x=unit(co.native2[,1], "native"), y=unit(co.native2[,2], "native"),
 					   size=unit(symbol.size2, "inch"),
 					   pch=symbol.shape2,
 					   gp=get_symbol_gpar(x=symbol.shape2,
@@ -449,49 +534,56 @@ plot_symbols <- function(co.npc, g, gt, lineInch, i, k) {
 	})
 }
 
-plot_text <- function(co.npc, g, gt, lineInch, just=c("center", "center"), bg.margin=.10) {
-	lineHnpc <- convertHeight(unit(lineInch, "inch"), "npc", valueOnly=TRUE) * gt$scale
-	lineWnpc <- convertWidth(unit(lineInch, "inch"), "npc", valueOnly=TRUE) * gt$scale
+plot_text <- function(co.native, g, gt, lineNatH, lineNatW, just=c("center", "center"), bbx = bbx, bg.margin=.10) {
+	lineHnative <- lineNatH * gt$scale
+	lineWnative <- lineNatW * gt$scale
 	
-	npol <- nrow(co.npc)
+	npol <- nrow(co.native)
 	with(g, {
 		if (!any(text_sel)) {
-			warning("No text to display. Check if all size values are smaller than lowerbound.size, or if all positions fall outside the plotting area.", call. = FALSE)
+			#warning("No text to display. Check if all size values are smaller than lowerbound.size, or if all positions fall outside the plotting area.", call. = FALSE)
 			return(NULL)
 		}
 		
-		co.npc[, 1] <- co.npc[, 1] + text.xmod * lineWnpc
-		co.npc[, 2] <- co.npc[, 2] + text.ymod * lineHnpc
+		co.native[, 1] <- co.native[, 1] + text.xmod * lineWnative
+		co.native[, 2] <- co.native[, 2] + text.ymod * lineHnative
+
 		
-		grobText <- textGrob(text[text_sel], x=unit(co.npc[text_sel,1], "npc"), y=unit(co.npc[text_sel,2], "npc"), just=just, gp=gpar(col=text.color[text_sel], cex=text.size[text_sel], fontface=text.fontface, fontfamily=text.fontfamily))
+		#grobText <- textGrob(text[text_sel], x=unit(co.native[text_sel,1], "native"), y=unit(co.native[text_sel,2], "native"), just=just, gp=gpar(col=text.color[text_sel], fontface=text.fontface, fontfamily=text.fontfamily))
+
+		# grobText <- pointsGrob(x=unit(co.native[text_sel,1], "native"), y=unit(co.native[text_sel,2], "native"))
+		
+		grobText <- textGrob(text[text_sel], x=unit(co.native[text_sel,1], "native"), y=unit(co.native[text_sel,2], "native"), just=just, gp=gpar(col=text.color[text_sel], cex=text.size[text_sel], fontface=text.fontface, fontfamily=text.fontfamily))
 		nlines <- rep(1, length(text))
 		
 		
-		lineH <- convertHeight(unit(text.size[text_sel], "lines"), "npc", valueOnly=TRUE)
-		lineW <- convertWidth(unit(text.size[text_sel], "lines"), "npc", valueOnly=TRUE)
+		lineH <- npc_to_native(convertHeight(unit(text.size[text_sel], "lines"), "native", valueOnly=TRUE), scale = bbx[c(2,4)])
+		lineW <- npc_to_native(convertWidth(unit(text.size[text_sel], "lines"), "native", valueOnly=TRUE), scale = bbx[c(1,3)])
 		
 		#		if (!is.na(text.bg.color)) {
 		
+		tGH <- npc_to_native(mapply(text[text_sel], text.size[text_sel], nlines[text_sel], FUN=function(x,y,z){
+			convertHeight(grobHeight(textGrob(x, gp=gpar(cex=y, fontface=text.fontface, fontfamily=text.fontfamily))),"native", valueOnly=TRUE) * z/(z-0.25)}, USE.NAMES=FALSE), scale = bbx[c(2,4)])
 		
-		tGH <- mapply(text[text_sel], text.size[text_sel], nlines[text_sel], FUN=function(x,y,z){
-			convertHeight(grobHeight(textGrob(x, gp=gpar(cex=y, fontface=text.fontface, fontfamily=text.fontfamily))),"npc", valueOnly=TRUE) * z/(z-0.25)}, USE.NAMES=FALSE)
+		tGW <- npc_to_native(mapply(text[text_sel], text.size[text_sel], FUN=function(x,y){
+			convertWidth(grobWidth(textGrob(x, gp=gpar(cex=y, fontface=text.fontface, fontfamily=text.fontfamily))),"native", valueOnly=TRUE)}, USE.NAMES=FALSE), scale = bbx[c(1,3)])
 		
-		tGW <- mapply(text[text_sel], text.size[text_sel], FUN=function(x,y){
-			convertWidth(grobWidth(textGrob(x, gp=gpar(cex=y, fontface=text.fontface, fontfamily=text.fontfamily))),"npc", valueOnly=TRUE)}, USE.NAMES=FALSE)
-		tGX <- grobText$x + unit(ifelse(just[1]=="left", (tGW * .5), 
-										ifelse(just[1]=="right", -(tGW * .5), 0)), "npc")
-		tGY <- grobText$y + unit(tGH * ifelse(just[2]=="top", -.5, 
-									   ifelse(just[2]=="bottom", .5, -.05)), "npc")
+
+		justx <- .5 - just[1]
+		justy <- .6 - just[2]
 		
-		tGH <- tGH + lineH * bg.margin
-		tGW <- tGW + lineW * bg.margin
+		tGX <- grobText$x + unit(tGW * justx, "native")
+		tGY <- grobText$y + unit(tGH * justy, "native")
+		
+		tGH <- unit(tGH + lineH * bg.margin, "native")
+		tGW <- unit(tGW + lineW * bg.margin, "native")
 		grobTextBG <- rectGrob(x=tGX, y=tGY, width=tGW, height=tGH, gp=gpar(fill=text.bg.color, col=NA))
 		# 		} else {
 		# 			grobTextBG <- NULL
 		# 		}
 		
 		if (text.shadow) {
-			grobTextSh <- textGrob(text[text_sel], x=unit(co.npc[text_sel,1]+lineW * .05, "npc"), y=unit(co.npc[text_sel,2]- lineH * .05, "npc"), just=just, gp=gpar(col=text.shadowcol[text_sel], cex=text.size[text_sel], fontface=text.fontface, fontfamily=text.fontfamily))
+			grobTextSh <- textGrob(text[text_sel], x=unit(co.native[text_sel,1]+lineW * .05, "native"), y=unit(co.native[text_sel,2]- lineH * .05, "native"), just=just, gp=gpar(col=text.shadowcol[text_sel], cex=text.size[text_sel], fontface=text.fontface, fontfamily=text.fontfamily))
 		} else {
 			grobTextSh <- NULL
 		}
@@ -500,55 +592,64 @@ plot_text <- function(co.npc, g, gt, lineInch, just=c("center", "center"), bg.ma
 	})
 }
 
-
-
-
+################!!!!! Functions below needed for Advanced text options !!!!####################
 
 .grob2Poly <- function(g) {
-	x <- convertX(g$x, unitTo = "npc", valueOnly = TRUE)
-	y <- convertY(g$y, unitTo = "npc", valueOnly = TRUE)
+	x <- convertX(g$x, unitTo = "native", valueOnly = TRUE)
+	y <- convertY(g$y, unitTo = "native", valueOnly = TRUE)
 	if (inherits(g, "rect")) {
-		w <- convertWidth(g$width, unitTo = "npc", valueOnly = TRUE)
-		h <- convertHeight(g$height, unitTo = "npc", valueOnly = TRUE)
+		w <- convertWidth(g$width, unitTo = "native", valueOnly = TRUE)
+		h <- convertHeight(g$height, unitTo = "native", valueOnly = TRUE)
 		x1 <- x - .5*w
 		x2 <- x + .5*w
 		y1 <- y - .5*h
 		y2 <- y + .5*h
 		polys <- mapply(function(X1, X2, Y1, Y2) {
-			Polygon(cbind(c(X1, X2, X2, X1, X1),
-						  c(Y2, Y2, Y1, Y1, Y2)))
+			st_polygon(list(cbind(c(X1, X2, X2, X1, X1),
+						  c(Y2, Y2, Y1, Y1, Y2))))
 		}, x1, x2, y1, y2, SIMPLIFY=FALSE)
+		st_union(st_sfc(polys))
 	} else if (inherits(g, "polygon")) {
-		xs <- split(x, g$id)	
-		ys <- split(y, g$id)	
-		polys <- mapply(function(X, Y) {
-			Polygon(cbind(X[c(1:4,1)],
-						  Y[c(1:4,1)]))
-		}, xs, ys, SIMPLIFY=FALSE)
-	} else return(NULL)
-	
-	ids <- 1:length(polys)
-	gUnaryUnion(SpatialPolygons(mapply(function(p, i) {
-		Polygons(list(p), ID=i)
-	}, polys, ids, SIMPLIFY=FALSE)))
+		xs <- split(x, g$id)
+		ys <- split(y, g$id)
+		
+		polys <- mapply(function(xi, yi) {
+			co <- cbind(xi, yi)
+			st_polygon(list(rbind(co, co[1,])))
+		}, xs, ys, SIMPLIFY = FALSE)
+		st_union(st_sfc(polys))
+	} # else return(NULL)
+
+	# ids <- 1:length(polys)
+	# gUnaryUnion(SpatialPolygons(mapply(function(p, i) {
+	# 	Polygons(list(p), ID=i)
+	# }, polys, ids, SIMPLIFY=FALSE)))
 }
 
-polylineGrob2Lines <- function(gL) {
+polylineGrob2sfLines <- function(gL) {
 	k <- length(gL)
-	
-	SpatialLines(lapply(1:k, function(i) {
-		g <- gL[[i]]
-		x <- convertX(g$x, "npc", valueOnly = TRUE)
-		y <- convertY(g$y, "npc", valueOnly = TRUE)
-		id <- factor(g$id)
-		xs <- split(x, f = id)
-		ys <- split(y, f = id)
-		ids <- levels(id)
+
+	multiLines <- lapply(gL, function(gLi) {
+		coords <- cbind(gLi$x, gLi$y)
 		
-		Lines(mapply(function(X, Y) {
-			Line(cbind(X,Y))	
-		}, xs, ys, SIMPLIFY=FALSE), ID=i)
-	}))
+		if (length(gLi$id.lengths) > 1) {
+			ids <- unlist(mapply(rep, 1:length(gLi$id.lengths), gLi$id.lengths))
+			coords <- mapply(cbind, split(as.numeric(gLi$x), ids), split(as.numeric(gLi$y), ids))
+			st_multilinestring(coords)
+		} else {
+			st_linestring(coords)
+		}
+	})
+	st_sf(geometry = st_sfc(multiLines))
+}
+
+npc_to_native <- function(x, scale) {
+	x * (scale[2] - scale[1])# + scale[1]
+}
+
+native_to_npc_to_native <- function(x, scale) {
+	#(x - scale[1]) / (scale[2] - scale[1])
+	(x) / (scale[2] - scale[1])
 }
 
 .rectGrob2pathGrob <- function(rg, angles) {
@@ -556,29 +657,28 @@ polylineGrob2Lines <- function(gL) {
 	y <- convertY(rg$y, "inch", valueOnly=TRUE)
 	w <- convertWidth(rg$width, "inch", valueOnly=TRUE)
 	h <- convertHeight(rg$height, "inch", valueOnly=TRUE)
-	
+
 	a <- atan2(h, w)
-	
-	as <- as.vector(sapply(a, function(a)c(a,pi-a, pi+a,-a)))
-	
+	as <- as.vector(vapply(a, function(a)c(a,pi-a, pi+a,-a), numeric(4)))
+
 	as2 <- as + rep(angles * pi / 180, each=4)
-	
+
 	dst <- rep(sqrt((w/2)^2+(h/2)^2), each=4)
-	
-	xs <- rep(x, each=4) + cos(as2) * dst	
-	ys <- rep(y, each=4) + sin(as2) * dst	
-	
+
+	xs <- rep(x, each=4) + cos(as2) * dst
+	ys <- rep(y, each=4) + sin(as2) * dst
+
 	xs2 <- convertX(unit(xs, "inch"), "npc")
 	ys2 <- convertY(unit(ys, "inch"), "npc")
-	
+
 	id <- rep(1:length(x), each=4)
-	
+
 	w2 <- w + (h-w) * abs(sin(angles*pi/180))
 	h2 <- h + (w-h) * abs(sin(angles*pi/180))
-	
+
 	w3 <- convertWidth(unit(w2, "inch"), "npc")
 	h3 <- convertHeight(unit(h2, "inch"), "npc")
-	
+
 	list(poly=polygonGrob(xs2, ys2, id=id, gp=rg$gp),
 		 rect=rectGrob(rg$x, rg$y, width = w3, height=h3))
 }
@@ -586,14 +686,14 @@ polylineGrob2Lines <- function(gL) {
 .get_direction_angle <- function(co) {
 	p1 <- co[1,]
 	p2 <- co[nrow(co),]
-	
+
 	a <- atan2(p2[2] - p1[2], p2[1] - p1[1]) * 180 / pi
 	if (a < 0) a <- a + 360
 	a
 }
 
 
-.editGrob <- function(tg, sel, shiftX, shiftY, angles) {  
+.editGrob <- function(tg, sel, shiftX, shiftY, angles) {
 	nt <- length(sel)
 	angles <- rep(angles, length.out=nt)
 	if (any(angles!=0)) {
@@ -601,17 +701,17 @@ polylineGrob2Lines <- function(gL) {
 			tg <- .rectGrob2pathGrob(tg, angles)$poly
 		}
 	}
-	tgx <- convertX(tg$x, "npc", valueOnly = TRUE)
-	tgy <- convertY(tg$y, "npc", valueOnly = TRUE)
-	
+	tgx <- convertX(tg$x, "native", valueOnly = TRUE)
+	tgy <- convertY(tg$y, "native", valueOnly = TRUE)
+
 	if (inherits(tg, "polygon")) {
 		sel4 <- rep(sel, each=4)
-		tg$x <- unit(tgx + rep(shiftX, each=4), "npc")[sel4]
-		tg$y <- unit(tgy + rep(shiftY, each=4), "npc")[sel4]
+		tg$x <- unit(tgx + rep(shiftX, each=4), "native")[sel4]
+		tg$y <- unit(tgy + rep(shiftY, each=4), "native")[sel4]
 		tg$id <- rep(1:sum(sel), each=4)
 	} else {
-		tg$x <- unit(tgx + shiftX, "npc")[sel]
-		tg$y <- unit(tgy + shiftY, "npc")[sel]
+		tg$x <- unit(tgx + shiftX, "native")[sel]
+		tg$y <- unit(tgy + shiftY, "native")[sel]
 		if (inherits(tg, "rect")) {
 			tg$height <- tg$height[sel]
 			tg$width <- tg$width[sel]
@@ -620,10 +720,8 @@ polylineGrob2Lines <- function(gL) {
 			tg$rot <- angles[sel]
 		}
 	}
-	
-	
 	tg$gp <- do.call("gpar", lapply(unclass(tg$gp)[names(tg$gp)!="font"], function(g) {
-		if (length(g)==nt) g[sel] else g	
+		if (length(g)==nt) g[sel] else g
 	}))
 	tg
 }
