@@ -12,7 +12,7 @@ check_aes_args <- function(g) {
 		if (!is.character(gpal)) stop("Palette should be a character", call.=FALSE)
 		if (length(gpal)==1) {
 			if (substr(gpal, 1, 1)=="-") gpal <- substr(gpal, 2, nchar(gpal))
-			if (!gpal %in% c(rownames(brewer.pal.info), "seq", "div", "cat") && !valid_colors(gpal)) stop("Invalid palette", call.=FALSE)
+			if (!gpal %in% c(rownames(tmap.pal.info), "seq", "div", "cat") && !valid_colors(gpal)) stop("Invalid palette", call.=FALSE)
 		} else {
 			if (!all(valid_colors(gpal))) stop("Invalid palette", call.=FALSE)
 		}
@@ -27,20 +27,32 @@ check_aes_args <- function(g) {
 }
 
 process_col_vector <- function(x, sel, g, gt, reverse) {
+	
 	values <- x
 	#textNA <- ifelse(any(is.na(values[sel])), g$textNA, NA)
 	#showNA <- if (is.na(g$showNA)) any(is.na(values[sel])) else FALSE
 	
 	x[!sel] <- NA
 	
+	attr(x, "sel") <- as.vector(attr(x, "sel")) & as.vector(sel)
+	
 	check_aes_args(g)
 	
-	if (is.numeric(x)) {
-		rng <- range(x, na.rm = TRUE)
-		if (abs(rng[2] - rng[1]) < 1e-9) {
-			warning("The value range is less than 1e-9", call. = FALSE)
-			x[!is.na(x)] <- round(rng[1], 9)
+	allNA <- all(is.na(x)) 
+	isNUM <- is.numeric(x)
+
+	if (isNUM) {
+		if (allNA) {
+			g$style <- "cat"
+		} else {
+			rng <- range(x, na.rm = TRUE)
+			if (abs(rng[2] - rng[1]) < 1e-9 && rng[1] != rng[2]) {
+				warning("The value range of the variable \"", g$col, "\" is less than 1e-9", call. = FALSE)
+				x[!is.na(x)] <- round(rng[1], 9)
+			}
 		}
+	} else if (allNA) {
+		g$style <- "cat"
 	}
 	
 	if (length(na.omit(unique(x)))==1 && g$style!="fixed") g$style <- "cat"
@@ -48,7 +60,7 @@ process_col_vector <- function(x, sel, g, gt, reverse) {
 	if (is.factor(x) || g$style=="cat") {
 		
 		if (is.null(g$palette)) {
-			palette.type <- ifelse(is.ordered(x), "seq", "cat")
+			palette.type <- ifelse(is.ordered(x) || (isNUM), "seq", "cat")
 			palette <- gt$aes.palette[[palette.type]] 
 		} else if (g$palette[1] %in% c("seq", "div", "cat")) {
 			palette.type <- g$palette[1]
@@ -58,12 +70,14 @@ process_col_vector <- function(x, sel, g, gt, reverse) {
 			palette.type <- palette_type(palette)
 		}
 		colsLeg <- cat2pal(x,
+						   var = g$col,
 						   palette = palette,
-						   auto.palette.mapping = g$auto.palette.mapping,
+						   stretch.palette = g$stretch.palette,
 						   contrast = g$contrast,
 						   colorNA = g$colorNA,
+						   colorNULL=g$colorNULL,
 						   legend.labels=g$labels,
-						   max_levels=g$max.categories,
+						   max_levels=gt$max.categories,
 						   legend.NA.text = g$textNA,
 						   showNA = g$showNA,
 						   process.colors=c(list(alpha=g$alpha), gt$pc),
@@ -83,12 +97,16 @@ process_col_vector <- function(x, sel, g, gt, reverse) {
 		} else if (g$palette[1] %in% c("seq", "div", "cat")) {
 			gt$aes.palette[[g$palette[1]]]
 		} else g$palette
-		colsLeg <- num2pal(x, g$n, style=g$style, breaks=g$breaks, 
+		colsLeg <- num2pal(x, 
+						   var = g$col,
+						   call = g$call,
+						   g$n, style=g$style, breaks=g$breaks, 
 						   interval.closure=g$interval.closure,
 						   palette = palette,
-						   auto.palette.mapping = g$auto.palette.mapping,
+						   midpoint = g$midpoint, #auto.palette.mapping = g$auto.palette.mapping,
 						   contrast = g$contrast, legend.labels=g$labels,
 						   colorNA=g$colorNA, 
+						   colorNULL=g$colorNULL,
 						   legend.NA.text = g$textNA,
 						   showNA = g$showNA,
 						   process.colors=c(list(alpha=g$alpha), gt$pc),
@@ -122,33 +140,61 @@ process_col_vector <- function(x, sel, g, gt, reverse) {
 }
 
 
-process_dtcol <- function(dtcol, sel=NA, g, gt, nx, npol, check_dens=FALSE, areas=NULL, areas_unit=NULL, reverse) {
+process_dtcol <- function(xname, dtcol, sel=NA, g, gt, nx, npol, areas=NULL, areas_unit=NULL, text = NULL, text_sel = NULL) {
+	## dtcol = matrix if direct colors are given
+	## dtcol = list in case of disjoint small multiples
+	## dtcol = vector in case of small multiples processed once (i.e. they share the legend)
+	
+	reverse <- g[[aname("legend.reverse", xname)]]
+	
+	## return as color matrix (object col)
+	if (is.na(sel[1])) sel <- rep(TRUE, npol * nx)
+	sel[is.na(sel)] <- TRUE
+	
+
+
+	
 	is.constant <- is.matrix(dtcol)
 	if (is.constant) {
 		col <- dtcol
+		col.neutral <- apply(col, 2, function(bc) na.omit(bc)[1])
+		
+		if (all(is.na(col.neutral))) {
+			col.neutral[is.na(col.neutral)] <- "#000000" #dummy for empty facets
+		} else {
+			col.neutral[is.na(col.neutral)] <- col.neutral[which(!is.na(col.neutral))[1]]	## impute first non-NA value (in case of !free.scales, the legend of the first facet is taken)
+		}
+
+		col[is.na(col)] <- g$colorNULL
 		legend.labels <- NA
 		legend.values <- NA
 		legend.palette <- NA
-		col.neutral <- apply(col, 2, function(bc) na.omit(bc)[1])
 		breaks <- NA
 		values <- NA
 		title_append <- ""
 	} else if (is.list(dtcol)) {
 		# multiple variables for col are defined
-		if (is.na(sel[1])) sel <- rep(TRUE, nx)
+		
 		gsc <- split_g(g, n=nx)
 		
-		title_append <- rep("", nx)
-		if (check_dens) {
-			isNum <- sapply(dtcol, is.numeric)
-			isDens <- sapply(gsc, "[[", "convert2density")
-			
-			dtcol[isNum & isDens] <- lapply(dtcol[isNum & isDens], function(d) {
-				d / areas
-			})
-			title_append[isNum & isDens] <- paste("per", areas_unit)
-		}
+		dtcol_title_append <- mapply(check_num_col, dtcol, gsc, SIMPLIFY = FALSE, MoreArgs = list(areas, areas_unit))
+		dtcol <- lapply(dtcol_title_append, "[[", "col")
+		title_append <- vapply(dtcol_title_append, "[[", character(1), "title_append")
+		# 
+		# if (check_dens) {
+		# 	isNum <- vapply(dtcol, is.numeric, logical(1))
+		# 	isDens <- vapply(gsc, function(g) {
+		# 		!is.null(g$convert2density) && g$convert2density
+		# 	}, logical(1))
+		# 	unis <- 
+		# 	
+		# 	dtcol[isNum & isDens] <- lapply(dtcol[isNum & isDens], function(d) {
+		# 		d / areas
+		# 	})
+		# 	title_append[isNum & isDens] <- paste("per", areas_unit, " ")
+		# }
 		
+		sel <- split(sel, f = rep(1L:nx, each = npol))
 		res <- mapply(process_col_vector, dtcol, sel, gsc, MoreArgs=list(gt=gt, reverse=reverse), SIMPLIFY=FALSE)
 		col <- sapply(res, function(r)r$cols)
 		legend.labels <- lapply(res, function(r)r$legend.labels)
@@ -157,18 +203,19 @@ process_dtcol <- function(dtcol, sel=NA, g, gt, nx, npol, check_dens=FALSE, area
 		col.neutral <- lapply(res, function(r)r$col.neutral)
 		breaks <- lapply(res, function(r)r$breaks)
 		values <- dtcol
+		
+		## remove legend from facets with empty selection (due to zero size or lwd)
+		noData <- !sapply(sel, function(s) any(s))
+		legend.labels[noData] <- NA
 	} else {
-		if (check_dens) {
-			if (is.numeric(dtcol) && g$convert2density) {
-				dtcol <- dtcol / areas
-				title_append <- paste("per", areas_unit)
-			} else {
-				title_append <- ""
-			}
-		} else title_append <- ""
+
+		dtcol_title_append <- check_num_col(dtcol, g, areas, areas_unit)
+		dtcol <- dtcol_title_append$col
+		title_append <- dtcol_title_append$title_append
+
 		
 		#if (is.na(sel[1])) sel <- TRUE
-		sel[is.na(sel)] <- TRUE
+		
 		
 		res <- process_col_vector(dtcol, sel, g, gt, reverse)
 		col <- matrix(res$cols, nrow=npol)
@@ -179,14 +226,70 @@ process_dtcol <- function(dtcol, sel=NA, g, gt, nx, npol, check_dens=FALSE, area
 		breaks <- res$breaks
 		values <- split(dtcol, rep(1:nx, each=npol))
 	}
+	
+	if (xname == "fill") {
+		legend.misc <- list(lwd=g$gborders$lwd, border.col=g$gborders$col)	
+	} else if (xname == "line.col") {
+		legend.misc <- list(line.legend.lty = g$lty, line.legend.alpha = g$alpha) # legend.lwd added later
+	} else if (xname == "symbol.col") {
+		legend.misc <- list(symbol.border.lwd=g$border.lwd, symbol.normal.size=g$legend.max.symbol.size) # symbol.border.col added later
+	} else if (xname == "raster") {
+		legend.misc <- list()
+		if (is.constant) {
+			#legend.palette <- as.list(col.neutral)
+			legend.palette <- lapply(as.data.frame(dtcol, stringsAsFactors = FALSE), unique)
+		}
+	} else if (xname == "text.col") {
+		if (is.list(dtcol)) {
+			gsc <- split_g(g, n=nx)
+		}
+		if (is.list(values)) {
+			# process legend text
+			legend.text <- mapply(function(txt, v, b, s, l, gsci) {
+				if (is.na(gsci$labels.text[1])) {
+					
+					if (is.na(b[1])) {
+						# categorical
+						nl <- nlevels(v)
+						ids <- as.integer(v)
+					} else {
+						# numeric
+						nl <- length(b) - 1
+						ids <- as.integer(cut(v, breaks=b, include.lowest = TRUE, right = FALSE))
+					}
+					ix <- sapply(1:nl, function(i)which(ids==i & s)[1])
+					if (length(l)==nl+1) {
+						ix <- c(ix, which(is.na(v))[1])
+					}
+					coltext <- txt[ix]
+					coltext[is.na(coltext)] <- "NA"
+					
+				} else {
+					if (length(gsci$labels.text) == length(l)-1) {
+						coltext <- c(gsci$labels.text, "NA")
+					} else {
+						coltext <- rep(gsci$labels.text, length.out=length(l))
+					}
+				}
+				coltext
+			}, as.data.frame(text, stringsAsFactors = FALSE), values, breaks, as.list(as.data.frame(text_sel)), if (is.list(legend.labels)) legend.labels else list(legend.labels), if (is.list(dtcol)) gsc else list(g), SIMPLIFY=FALSE)
+		} else {
+			legend.text <- NA
+		}
+		legend.misc <- list(legend.text = legend.text)
+	}
+	
+
+	nonemptyFacets <- if (is.constant) NULL else if(is.list(values)) sapply(values, function(v) !all(is.na(v))) else rep(TRUE, nx)
 	list(is.constant=is.constant,
 		 col=col,
 		 legend.labels=legend.labels,
 		 legend.values=legend.values,
 		 legend.palette=legend.palette,
 		 col.neutral=col.neutral,
-		 breaks=breaks,
-		 values=values,
+		 legend.misc = legend.misc,
+		 legend.hist.misc=list(values=values, breaks=breaks, densities=g$convert2density),
+		 nonemptyFacets=nonemptyFacets,
 		 title_append=title_append)
 }
 
