@@ -1,35 +1,33 @@
 #' Layout of cartographic maps
 #' 
-#' This element specifies the map layout. The main function \code{tm_layout} controls title, margins, aspect ratio, colors, frame, legend, among many other things. The function \code{tm_legend} is a shortcut to access all \code{legend.} arguments without this prefix. The other functions are wrappers for two purposes: the \code{tm_format_} functions specify position related layout settings such as margins, and the \code{tm_style_} functions specify general styling related layout settings such as colors and font. Typically, the former functions are shape dependent, and the latter functions are shape independent. See details for predefined styles and formats. With the global option \code{tmap.style}, a default style can be specified. Multiple \code{tm_layout} elements (or wrapper functions) can be stacked: called arguments will be overwritten.
+#' This element specifies the map layout. The main function \code{tm_layout} controls title, margins, aspect ratio, colors, frame, legend, among many other things. The function \code{tm_legend} is a shortcut to access all \code{legend.} arguments without this prefix. The other functions are wrappers for two purposes: \code{tm_format} specifies position related layout settings such as margins, and \code{tm_style} specifies general styling related layout settings such as colors and font. Typically, the former functions are shape dependent, and the latter functions are shape independent. See details for predefined styles and formats. With \code{tmap.style}, a default style can be specified. Multiple \code{tm_layout} elements (or wrapper functions) can be stacked: called arguments will be overwritten.
 #' 
 #' Predefined styles:
 #' \tabular{ll}{
-#' \code{tm_style_white}\tab White background, commonly used colors (default) \cr
-#' \code{tm_style_gray}/\code{_grey}\tab Grey background, useful to highlight sequential palettes (e.g. in choropleths) \cr
-#' \code{tm_style_natural}\tab Emulation of natural view: blue waters and green land \cr
-#' \code{tm_style_bw}\tab Greyscale, obviously useful for greyscale printing \cr
-#' \code{tm_style_classic}\tab Classic styled maps (recommended) \cr
-#' \code{tm_style_cobalt}\tab Inspired by latex beamer style cobalt \cr
-#' \code{tm_style_albatross}\tab Inspired by latex beamer style albatross \cr
-#' \code{tm_style_beaver}\tab Inspired by latex beamer style beaver \cr
+#' \code{"white"}\tab White background, commonly used colors (default) \cr
+#' \code{"gray"}/\code{"grey"}\tab Grey background, useful to highlight sequential palettes (e.g. in choropleths) \cr
+#' \code{"natural"}\tab Emulation of natural view: blue waters and green land \cr
+#' \code{"bw"}\tab Greyscale, obviously useful for greyscale printing \cr
+#' \code{"classic"}\tab Classic styled maps (recommended) \cr
+#' \code{"cobalt"}\tab Inspired by latex beamer style cobalt \cr
+#' \code{"albatross"}\tab Inspired by latex beamer style albatross \cr
+#' \code{"beaver"}\tab Inspired by latex beamer style beaver \cr
 #' --------------------------- \tab --------------------------------------------------------------------------------------------------- \cr
 #' }
 #'
 #' Predefined formats
 #' \tabular{ll}{
-#' \code{tm_format_World}\tab Format specified for world maps \cr
-#' \code{tm_format_World_wide}\tab Format specified for world maps with more space for the legend \cr
-#' \code{tm_format_Europe}\tab Format specified for maps of Europe \cr
-#' \code{tm_format_Europe_wide}\tab Format specified for maps of Europe with more space for the legend \cr
-#' \code{tm_format_NLD}\tab Format specified for maps of the Netherlands \cr
-#' \code{tm_format_NLD_wide}\tab Format specified for maps of the Netherlands with more space for the legend \cr
+#' \code{"World"}\tab Format specified for world maps \cr
+#' \code{"World_wide"}\tab Format specified for world maps with more space for the legend \cr
+#' \code{"NLD"}\tab Format specified for maps of the Netherlands \cr
+#' \code{"NLD_wide"}\tab Format specified for maps of the Netherlands with more space for the legend \cr
 #' --------------------------- \tab --------------------------------------------------------------------------------------------------- \cr
 #' }
 #' 
 #' @name tm_layout
 #' @rdname tm_layout
 #' @param title Global title of the map. For small multiples, multiple titles can be specified. The title is drawn inside the map. Alternatively, use \code{panel.labels} to print the map as a panel, with the title inside the panel header (especially useful for small multiples). Another alternative is the \code{main.title} which prints a title above the map. Titles for the legend items are specified at the layer functions (e.g. \code{\link{tm_fill}}).
-#' @param scale numeric value that serves as the global scale parameter. All font sizes, symbol sizes, border widths, and line widths are controled by this value. Each of these elements can be scaled independently with the \code{scale}, \code{lwd}, or \code{size} arguments provided by the \code{\link{tmap-element}s}.
+#' @param scale numeric value that serves as the global scale parameter. All font sizes, symbol sizes, border widths, and line widths are controlled by this value. Each of these elements can be scaled independently with the \code{scale}, \code{lwd}, or \code{size} arguments provided by the \code{\link{tmap-element}s}.
 #' @param title.size Relative size of the title
 #' @param bg.color Background color. By default it is \code{"white"}. A recommended alternative for choropleths is light grey (e.g., \code{"grey85"}).
 #' @param aes.color Default color values for the aesthetics layers. Should be a named vector with the names chosen from: \code{fill}, \code{borders}, \code{symbols}, \code{dots}, \code{lines}, \code{text}, \code{na}. Use \code{"#00000000"} for transparency.
@@ -48,7 +46,7 @@
 #' @param fontface font face of all text in the map.
 #' @param fontfamily font family of the text labels.
 #' @param compass.type type of compass, one of: \code{"arrow"}, \code{"4star"}, \code{"8star"}, \code{"radar"}, \code{"rose"}. Of course, only applicable if a compass is shown. The compass type can also be set within \code{\link{tm_compass}}.
-#' @param earth.boundary Logical that determines whether the boundaries of the earth are shown or an object that specifies the boundaries. This object can be a vector of size four, a 2 by 2 matrix (bounding box), or an \code{\link[raster:extent]{extent}} object. By default, the boundaries are \code{c(-180, 180, -90, 90)}. Useful for projected world maps. Often, it is useful to crop both poles (e.g., with \code{c(-180, 180, -88, 88)}).
+#' @param earth.boundary Logical that determines whether the boundaries of the earth are shown or an object that specifies the boundaries. This object can be a vector of size four, a 2 by 2 matrix (bounding box), or an \code{\link[raster:extent]{extent}} object. By default, the boundaries are \code{c(-180, -90, 180, 90)}. Useful for projected world maps. Often, it is useful to crop both poles (e.g., with \code{c(-180, -88, 180, 88)}).
 #' @param earth.boundary.color Color of the earth boundary.
 #' @param earth.boundary.lwd Line width of the earth boundary.
 #' @param earth.datum Geodetic datum to determine the earth boundary. By default \code{"WGS84"}, other frequently used datums are \code{"NAD83"} and \code{"NAD27"}. Any other \code{PROJ.4} character string can be used.
@@ -58,7 +56,7 @@
 #' @param legend.outside Logical that determines whether the legend is plot outside of the map/facets. Especially useful when using facets that have a common legend (i.e. with \code{free.scales=FALSE}).
 #' @param legend.outside.position Character that determines the outside position of the legend. Only applicable when \code{legend.outside=TRUE}. One of: \code{"right"}, \code{"left"}, \code{"top"}, or \code{"bottom"}.
 #' @param legend.outside.size Numeric value that determines the relative size of the legend, when \code{legend.outside=TRUE}. If the first value of \code{legend.outside.position} is \code{"top"} or \code{"bottom"}, then it is the width of the legend, else it is the height of the legend.
-#' @param legend.position Position of the legend. Vector of two values, specifing the x and y coordinates. Either this vector contains \code{"left"}, \code{"LEFT"}, \code{"center"}, \code{"right"}, or \code{"RIGHT"} for the first value and \code{"top"}, \code{"TOP"}, \code{"center"}, \code{"bottom"}, or \code{"BOTTOM"} for the second value, or this vector contains two numeric values between 0 and 1 that specifies the x and y coordinates of the left bottom corner of the legend. The uppercase values correspond to the position without margins (so tighter to the frame). By default, it is automatically placed in the corner with most space based on the (first) shape object. If \code{legend.outside=TRUE}, this argument specifies the legend position within the outside panel.
+#' @param legend.position Position of the legend. Vector of two values, specifying the x and y coordinates. Either this vector contains \code{"left"}, \code{"LEFT"}, \code{"center"}, \code{"right"}, or \code{"RIGHT"} for the first value and \code{"top"}, \code{"TOP"}, \code{"center"}, \code{"bottom"}, or \code{"BOTTOM"} for the second value, or this vector contains two numeric values between 0 and 1 that specifies the x and y coordinates of the left bottom corner of the legend. The uppercase values correspond to the position without margins (so tighter to the frame). By default, it is automatically placed in the corner with most space based on the (first) shape object. If \code{legend.outside=TRUE}, this argument specifies the legend position within the outside panel.
 #' @param legend.stack Value that determines how the legend items are stacked: \code{"vertical"} or \code{"horizontal"}.
 #' @param legend.just Justification of the legend relative to the point coordinates.  The first value specifies horizontal and the second value vertical justification. Possible values are: \code{"left"} , \code{"right"}, \code{"center"}, \code{"bottom"}, and \code{"top"}. Numeric values of 0 specify left/bottom alignment and 1 right/top alignment. This option is only used, if \code{legend.position} is specified by numeric coordinates.
 #' @param legend.width width of the legend. If it is a negative number, it will be the exact legend width. If it is a positive number (by default), it will be the maximum legend width; the actual legend width will be decreased automatically based on the legend content and font sizes.
@@ -89,10 +87,11 @@
 #' @param legend.hist.bg.color Background color of the histogram
 #' @param legend.hist.bg.alpha Transparency number between 0 (totally transparent) and 1 (not transparent). By default, the alpha value of the \code{legend.hist.bg.color} is used (normally 1).
 #' @param title.snap.to.legend Logical that determines whether the title is part of the legend. By default \code{FALSE}, unless the legend is drawn outside the map (see \code{legend.outside}).
-#' @param title.position Position of the title. Vector of two values, specifing the x and y coordinates. Either this vector contains "left", "LEFT", "center", "right", or "RIGHT" for the first value and "top", "TOP", "center", "bottom", or "BOTTOM" for the second value, or this vector contains two numeric values between 0 and 1 that specifies the x and y coordinates of the tile. The uppercase values correspond to the position without margins (so tighter to the frame). 
+#' @param title.position Position of the title. Vector of two values, specifying the x and y coordinates. Either this vector contains "left", "LEFT", "center", "right", or "RIGHT" for the first value and "top", "TOP", "center", "bottom", or "BOTTOM" for the second value, or this vector contains two numeric values between 0 and 1 that specifies the x and y coordinates of the tile. The uppercase values correspond to the position without margins (so tighter to the frame). 
 #' By default the title is placed on top of the legend (determined by \code{legend.position}).
 #' @param title.color color of the title
-#' @param legend.frame either a logical that determines whether the legend is placed inside a frame, or a color that directly specifies the frame border color. The width of the frame is automatically determined, but is upper-bounded by \code{legend.width}.
+#' @param legend.frame either a logical that determines whether the legend is placed inside a frame, or a color that directly specifies the frame border color.
+#' @param legend.frame.lwd line width of the legend frame (applicable if \code{legend.frame} is \code{TRUE} or a color)
 #' @param title.bg.color background color of the title. Use \code{TRUE} to match with the overall background color \code{bg.color}. By default, it is \code{TRUE} if \code{legend.frame} is \code{TRUE} or a color.
 #' @param title.bg.alpha Transparency number between 0 (totally transparent) and 1 (not transparent). By default, the alpha value of the \code{title.bg.color} is used (normally 1).
 #' @param panel.show Logical that determines if the map(s) are shown as panels. If \code{TRUE}, the title will be placed in the panel header instead of inside the map. By default, it is \code{TRUE} when small multiples are created with the \code{by} variable. (See \code{\link{tm_facets}}) 
@@ -109,98 +108,103 @@
 #' @param attr.outside Logical that determines whether the attributes are plot outside of the map/facets.
 #' @param attr.outside.position Character that determines the outside position of the attributes: \code{"top"} or \code{"bottom"}. Only applicable when \code{attr.outside=TRUE}. If the legend is also drawn outside (with \code{legend.outside=TRUE}) and on the same side of the map (e.g. also \code{"top"} or \code{"bottom"}), the attributes are placed between the map and the legend. This can be changed by setting \code{attr.outside.position} to \code{"TOP"} or \code{"BOTTOM"}: in this case, the attributes are placed above respectively below the legend.
 #' @param attr.outside.size Numeric value that determines the relative height of the attribute viewport, when \code{attr.outside=TRUE}.
-#' @param attr.position Position of the map attributes, which are \code{\link{tm_credits}}, \code{\link{tm_scale_bar}} and \code{\link{tm_compass}}. Vector of two values, specifing the x and y coordinates. The first value is \code{"left"}, \code{"LEFT"}, \code{"center"}, \code{"right"}, or \code{"RIGHT"}, and the second value \code{"top"}, \code{"TOP"}, \code{"center"}, \code{"bottom"}, or \code{"BOTTOM"}. The uppercase values correspond to the position without margins (so tighter to the frame). Positions can also be set separately in the map attribute fuctions. If \code{attr.outside=TRUE}, this argument specifies the position of the attributes within the outside panel.
+#' @param attr.position Position of the map attributes, which are \code{\link{tm_credits}}, \code{\link{tm_scale_bar}}, \code{\link{tm_compass}}, and \code{\link{tm_minimap}}. Vector of two values, specifying the x and y coordinates. The first value is \code{"left"}, \code{"LEFT"}, \code{"center"}, \code{"right"}, or \code{"RIGHT"}, and the second value \code{"top"}, \code{"TOP"}, \code{"center"}, \code{"bottom"}, or \code{"BOTTOM"}. The uppercase values correspond to the position without margins (so tighter to the frame). Positions can also be set separately in the map attribute functions. If \code{attr.outside=TRUE}, this argument specifies the position of the attributes within the outside panel.
 #' @param attr.just Justification of the attributes relative to the point coordinates.  The first value specifies horizontal and the second value vertical justification. Possible values are: \code{"left"} , \code{"right"}, \code{"center"}, \code{"bottom"}, and \code{"top"}. Numeric values of 0 specify left/bottom alignment and 1 right/top alignment. This option is only used, if \code{attr.position} is specified by numeric coordinates. It can also be specified per attribute function.
 #' @param design.mode Logical that enables the design mode. If \code{TRUE}, inner and outer margins, legend position, aspect ratio are explicitly shown. Also, feedback text in the console is given.
-#' @param basemaps vector of one or more names of baselayer maps used in the interactive view mode. See \code{\link{tm_view}}.
-#' @param basemaps.alpha alpha transparency (opacity) of the basemaps. See \code{\link{tm_view}}.
+#' @param basemaps Migrated to \code{\link{tm_view}}. vector of one or more names of baselayer maps used in the interactive view mode. See \code{\link{tm_view}}.
+#' @param basemaps.alpha Migrated to \code{\link{tm_view}}. alpha transparency (opacity) of the basemaps. See \code{\link{tm_view}}.
 #' @param bg.overlay Not used anymore. See \code{\link{tm_view}}.
 #' @param bg.overlay.alpha Not used anymore. See \code{\link{tm_view}}.
 #' @param ... other arguments from \code{tm_layout}
-#' @seealso \href{../doc/tmap-nutshell.html}{\code{vignette("tmap-nutshell")}}
+#' @seealso \href{../doc/tmap-getstarted.html}{\code{vignette("tmap-getstarted")}}
 #' @references Tennekes, M., 2018, {tmap}: Thematic Maps in {R}, Journal of Statistical Software, 84(6), 1-39, \href{https://doi.org/10.18637/jss.v084.i06}{DOI}
 #' @example ./examples/tm_layout.R
 #' @export
-tm_layout <- function(title=NA,
-					  scale=1,
-					  title.size=1.3,
-					  bg.color= "white",
-					  aes.color=c(fill="grey85", borders="grey40", symbols="grey60", dots="black", lines="red", text="black", na="grey75"),
-					  aes.palette=list(seq="YlOrBr", div="RdYlGn", cat="Set3"),
-					  attr.color="black",
-  					  sepia.intensity=0, 
-  					  saturation=1, 
-					  frame=TRUE,
-  					  frame.lwd=1,
-  					  frame.double.line=FALSE,
-					  asp = NA,
-					  outer.margins = rep(0.02, 4),
-					  inner.margins = NA,
-					  between.margin = .5,
-					  outer.bg.color=NULL,
-					  fontface="plain", 
-					  fontfamily="",
- 					  compass.type="arrow",
-					  earth.boundary=FALSE,
-					  earth.boundary.color=attr.color,
-					  earth.boundary.lwd=1,
-					  earth.datum="WGS84",
-					  space.color=NULL,
-					  legend.show = TRUE,
-					  legend.only = FALSE,
-					  legend.outside=NA,
-					  legend.outside.position="right",
-					  legend.outside.size=0.3,
-					  legend.position = NULL,
-					  legend.stack = c("vertical", "horizontal"),
-					  legend.just = c("left", "bottom"),
-					  legend.width = 0.4,
-					  legend.height = 0.9,
-					  legend.hist.height = 0.3,
-					  legend.hist.width = legend.width,
-					  legend.title.size=1.1,
-					  legend.text.size=0.7,
-					  legend.hist.size=0.7,
-					  legend.format=list(fun=NULL, scientific = FALSE, digits= NA, 
-					  				   text.separator = "to", 
-					  				   text.less.than = c("Less", "than"),
-					  				   text.or.more = c("or", "more"),
-					  				   text.align = NA, 
-					  				   text.to.columns = FALSE),
-					  legend.frame = FALSE,
-					  legend.text.color = attr.color,
-					  legend.bg.color = NA,
-					  legend.bg.alpha = 1,
-					  legend.hist.bg.color = NA,
-					  legend.hist.bg.alpha = 1,
-					  title.snap.to.legend = NA,
-					  title.position = c("left", "top"),
-					  title.color=attr.color,
-					  title.bg.color=NA,
-					  title.bg.alpha = 1,
-					  panel.show = NA,
-					  panel.labels=NA,
-					  panel.label.size = 1,
-					  panel.label.color = "black",
-					  panel.label.bg.color = "grey80",
-					  panel.label.height = 1.25,
-					  panel.label.rot = c(90, 0),
-					  main.title = NA,
-					  main.title.size = 1.5,
-					  main.title.color = "black",
-					  main.title.position = "left",
-					  attr.outside = FALSE,
-					  attr.outside.position = "bottom",
-					  attr.outside.size=NA,
-					  attr.position = c("right", "bottom"),
-					  attr.just = c("left", "bottom"),
-					  design.mode = FALSE,
-					  basemaps = c("CartoDB.Positron", "OpenStreetMap", "Esri.WorldTopoMap"),
-					  basemaps.alpha = c(1, 1, 1),
-					  bg.overlay=NULL,
-					  bg.overlay.alpha=0) {
-	legend.stack <- match.arg(legend.stack)
-	g <- list(tm_layout=c(as.list(environment()), list(call=names(match.call(expand.dots = TRUE)[-1]))))
+tm_layout <- function(title,
+					  scale,
+					  title.size,
+					  bg.color,
+					  aes.color,
+					  aes.palette,
+					  attr.color,
+  					  sepia.intensity, 
+  					  saturation, 
+					  frame,
+  					  frame.lwd,
+  					  frame.double.line,
+					  asp,
+					  outer.margins,
+					  inner.margins,
+					  between.margin,
+					  outer.bg.color,
+					  fontface, 
+					  fontfamily,
+ 					  compass.type,
+					  earth.boundary,
+					  earth.boundary.color,
+					  earth.boundary.lwd,
+					  earth.datum,
+					  space.color,
+					  legend.show,
+					  legend.only,
+					  legend.outside,
+					  legend.outside.position,
+					  legend.outside.size,
+					  legend.position,
+					  legend.stack,
+					  legend.just,
+					  legend.width,
+					  legend.height,
+					  legend.hist.height,
+					  legend.hist.width,
+					  legend.title.size,
+					  legend.text.size,
+					  legend.hist.size,
+					  legend.format,
+					  legend.frame,
+					  legend.frame.lwd,
+					  legend.text.color,
+					  legend.bg.color,
+					  legend.bg.alpha,
+					  legend.hist.bg.color,
+					  legend.hist.bg.alpha,
+					  title.snap.to.legend,
+					  title.position,
+					  title.color,
+					  title.bg.color,
+					  title.bg.alpha,
+					  panel.show,
+					  panel.labels,
+					  panel.label.size,
+					  panel.label.color,
+					  panel.label.bg.color,
+					  panel.label.height,
+					  panel.label.rot,
+					  main.title,
+					  main.title.size,
+					  main.title.color,
+					  main.title.position,
+					  attr.outside,
+					  attr.outside.position,
+					  attr.outside.size,
+					  attr.position,
+					  attr.just,
+					  design.mode,
+					  basemaps,
+					  basemaps.alpha,
+					  bg.overlay,
+					  bg.overlay.alpha) {
+
+	e1 <- parent.frame()
+	args <- lapply(as.list(match.call()[-1]), eval, envir = e1)
+	args$style <- NA
+	
+	if (any(c("basemaps", "basemaps.alpha") %in% names(args))) warning("As of version 2.0, basemaps and basemaps.alpha have to be called from tm_view")
+	if (any(c("bg.overlay", "bg.overlay.alpha") %in% names(args))) warning("bg.overlay and bg.overlay.alpha are not used anymore")
+			
+	args[c("bg.overlay", "bg.overlay.alpha")] <- NULL
+
+	g <- list(tm_layout=args)
 	class(g) <- "tm"
 	g
 }
@@ -219,232 +223,40 @@ tm_legend <- function(...) {
 	do.call("tm_layout", x)
 }
 
-#' @rdname tm_layout
-#' @export
-tm_format_World <- function(title=NA,
-							inner.margins=c(0, 0.05, 0.025, 0.01),
-							legend.position=c("left", "bottom"), 
-							attr.position=c("right", "bottom"),
-							scale=.8,
-							...) {
-	args <- c(as.list(environment()), list(...))
-	do.call("tm_layout", args)
-}
-
 
 #' @rdname tm_layout
+#' @param style name of the style
 #' @export
-tm_format_World_wide <- function(title=NA,
-								 inner.margins=c(0, 0.2, 0.025, 0.01),
-							legend.position=c("left", "bottom"), 
-							attr.position=c("right", "bottom"),
-							scale=.8,
-							...) {
-	args <- c(as.list(environment()), list(...))
-	do.call("tm_layout", args)
-}
-
-#' @rdname tm_layout
-#' @export
-tm_format_Europe <- function(title=NA,
-							 title.position=c("left", "top"),
-							 legend.position=c("right", "top"),
-							 legend.just=c("right", "top"),
-							 attr.position=c("right", "bottom"),
-							 legend.frame=TRUE,
-							 inner.margins=c(0, 0, 0, 0),
-							 
-							 ...) {
-	args <- c(as.list(environment()), list(...))
-	do.call("tm_layout", args)
-}
-
-#' @rdname tm_layout
-#' @export
-tm_format_Europe2 <- function(title=NA,
-							  title.position=c("left", "top"),
-							  legend.position=c("left", "top"),
-							  legend.outside=TRUE,
-							  legend.outside.size=.2,
-							  legend.just=c("right", "top"),
-							  attr.position=c("right", "bottom"),
-							  inner.margins=c(0, 0, 0, 0),
-							  ...) {
-	args <- c(as.list(environment()), list(...))
-	do.call("tm_layout", args)
-}
-
-#' @rdname tm_layout
-#' @export
-tm_format_Europe_wide <- function(title=NA,
-								  title.position=c("left",.85),
-								  title.snap.to.legend=TRUE,
-							 legend.position=c("left", .85), 
-							 legend.just=c("left", "top"),
-							 attr.position=c("left", "bottom"),
-							 inner.margins=c(0, 0.25, 0, 0),
-							 ...) {
-	args <- c(as.list(environment()), list(...))
-	do.call("tm_layout", args)
-}
-
-
-
-#' @rdname tm_layout
-#' @export
-tm_format_NLD <- function(title=NA,
-						  frame=FALSE, 
-						  inner.margins=c(.02, .2, .06, .02),
-						  legend.position=c("left", "top"), 
-						  attr.position=c("left", "bottom"),
-						  ...) {
-	args <- c(as.list(environment()), list(...))
-	do.call("tm_layout", args)
-}
-
-#' @rdname tm_layout
-#' @export
-tm_format_NLD_wide <- function(title=NA,
-						  frame=FALSE, 
-						  inner.margins=c(.02, .3, .06, .02),
-						  legend.position=c("left", "top"), 
-						  attr.position=c("left", "bottom"),
-						  ...) {
-	args <- c(as.list(environment()), list(...))
-	do.call("tm_layout", args)
-}
-
-style_args <- c("bg.color", "aes.color", "aes.palette", "attr.color", "saturation", "sepia.intensity", "fontfamily", "frame.double.line", "compass.type", "space.color")
-
-#' @rdname tm_layout
-#' @export
-tm_style_white <- function(...) {
+tm_style <- function(style, ...) {
 	args <- list(...)
-	g <- do.call("tm_layout", args)
-	g$tm_layout$call <- union(g$tm_layout$call, style_args)
-	g
-}
-
-#' @rdname tm_layout
-#' @export
-tm_style_gray <- function(bg.color="grey85", 
-						   aes.color=c(fill="grey70", borders="grey20", symbols="grey50", dots="black", lines="red", text="black", na="grey60"),
-						   ...) {
-	args <- c(as.list(environment()), list(...))
-	g <- do.call("tm_layout", args)
-	g$tm_layout$call <- union(g$tm_layout$call, style_args)
-	g
-}
-
-#' @rdname tm_layout
-#' @export
-tm_style_natural <- function(bg.color="lightskyblue1",
-							 aes.color=c(fill="darkolivegreen3", borders="black", symbols="tomato2", dots="firebrick", lines="steelblue", text="black", na="white"),
-							 aes.palette=list(seq="YlGn", div="RdYlGn", cat="Set3"),
-							 attr.color="black", 
-							 space.color="white",
-							 legend.frame=TRUE,
-							 legend.bg.color="grey90",
-							 earth.boundary=TRUE,
-							 basemaps="Esri.NatGeoWorldMap",
-							 basemaps.alpha=1,
-							 ...) {
-
-	args <- c(as.list(environment()), list(...))
-	g <- do.call("tm_layout", args)
-	g$tm_layout$call <- union(g$tm_layout$call, style_args)
-	g
-}
-
-#' @rdname tm_layout
-#' @export
-tm_style_grey <- tm_style_gray
-
-#' @rdname tm_layout
-#' @export
-tm_style_cobalt <- function(bg.color="#002240",
-							aes.color=c(fill="#0088FF", borders="#002240", symbols="#FF9D00", dots="#FF9D00", lines="#FFEE80", text="white", na="grey60"),
-							aes.palette=list(seq="YlGn", div="RdYlGn", cat="Set3"),
-							attr.color="white", 
-							basemaps="CartoDB.DarkMatter",
-							basemaps.alpha=.5,
-							...) {
-	# Bu="#0088FF" DaBu="#002240" LiBu="#BED6FF" Or="#FF9D00", W="white" Yl="FFEE80"
-	# See https://www.hartwork.org/beamer-theme-matrix/
 	
-	args <- c(as.list(environment()), list(...))
-	g <- do.call("tm_layout", args)
-	g$tm_layout$call <- union(g$tm_layout$call, style_args)
-	g
-}
-
-#' @rdname tm_layout
-#' @export
-tm_style_col_blind <- function(bg.color="white",
-							aes.color=c(fill="grey85", borders="black", symbols="#D55E00", dots="#0072B2", lines="#009E73", text="black", na="white"),
-							aes.palette=list(seq="Blues", div="RdBu", cat=c("#D55E00", "#56B4E9", "#E69F00", "#009E73", "#F0E442", "#0072B2","#CC79A7")),
-							attr.color="black", ...) {
-
-	args <- c(as.list(environment()), list(...))
-	g <- do.call("tm_layout", args)
-	g$tm_layout$call <- union(g$tm_layout$call, style_args)
-	g
-}
-
-
-#' @rdname tm_layout
-#' @export
-tm_style_albatross <- function(bg.color="#00007F",
-								aes.color=c(fill="#4C4C88", borders="#00004C", symbols="#BFBFFF", dots="#BFBFFF", lines="#BFBFFF", text="#FFE700", na="grey60"),
-								aes.palette=list(seq="YlOrRd", div="RdYlGn", cat="Set3"),
-								attr.color="#BFBFFF",
-								basemaps="CartoDB.DarkMatter",
-								basemaps.alpha=.5,
-								...) {
-	# Y="#FFE700", Bu="#00007F", DaBu="#00004C", Gr="#BFBFFF", DaGr="#4C4C88"
-	# See https://www.hartwork.org/beamer-theme-matrix/
+	.tmapOptions <- get(".tmapOptions", envir = .TMAP_CACHE)	
+	check_style(style)
 	
-	args <- c(as.list(environment()), list(...))
-	g <- do.call("tm_layout", args)
-	g$tm_layout$call <- union(g$tm_layout$call, style_args)
-	g
+	args$style <- style
+	structure(list(tm_layout=args), class = "tm")
 }
 
 #' @rdname tm_layout
+#' @param format name of the format
 #' @export
-tm_style_beaver <- function(bg.color="#FFFFFF",
-							 aes.color=c(fill="#FFE200", borders="#000000", symbols="#A30000", dots="#A30000", lines="#A30000", text="#000000", na="#E0E0E0"),
-							 aes.palette=list(seq="YlOrBr", div="RdYlGn", cat="Dark2"),
-							 attr.color="black", ...) {
-	# W="#FFFFFF" Y="#FFE200" Rd="#A30000" Bl="#000000" LiGr="#F0F0F0" Gr="E0E0E0"
-	# See https://www.hartwork.org/beamer-theme-matrix/
+tm_format <- function(format, ...) {
+	args <- list(...)
 	
-	args <- c(as.list(environment()), list(...))
-	g <- do.call("tm_layout", args)
-	g$tm_layout$call <- union(g$tm_layout$call, style_args)
-	g
+	.tmapFormats <- get(".tmapFormats", envir = .TMAP_CACHE)
+	
+	if (!(format %in% names(.tmapFormats))) stop("Unknown format. Please check tmap_format() for available formats")
+	
+	formatArgs <- .tmapFormats[[format]]
+	if (length(args)) {
+		formatArgs[names(args)] <- args	
+	}
+	formatArgs$style <- NA
+	
+	called <- names(args)
+	if (is.null(called)) called <- character(0)
+	
+	attr(formatArgs, "format_args") <- called
+	structure(list(tm_layout=formatArgs), class = "tm")
 }
 
-#' @rdname tm_layout
-#' @export
-tm_style_bw <- function(saturation=0, ...) {
-	args <- c(as.list(environment()), list(...))
-	g <- do.call("tm_layout", args)
-	g$tm_layout$call <- union(g$tm_layout$call, style_args)
-	g
-}
-
-#' @rdname tm_layout
-#' @export
-tm_style_classic <- function(sepia.intensity=.7, 
-							 fontfamily="serif", 
-							 frame.double.line=TRUE, 
-							 compass.type="rose",
-							 basemaps="Esri.WorldTopoMap",
-							 basemaps.alpha=.5,
-							 ...) {
-	args <- c(as.list(environment()), list(...))
-	g <- do.call("tm_layout", args)
-	g$tm_layout$call <- union(g$tm_layout$call, style_args)
-	g
-}
