@@ -1,4 +1,4 @@
-view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
+view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE, gal = NULL) {
 	
 	# determine view
 
@@ -8,8 +8,8 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 	gp$tm_layout <- NULL
 	
 	lf <- leaflet(options = leaflet::leafletOptions(crs=gt$projection))
-	basemaps <- gt$basemaps
-	basemaps.alpha <- gt$basemaps.alpha
+	# basemaps <- gt$basemaps
+	# basemaps.alpha <- gt$basemaps.alpha
 	
 	pOptions <- function(cw) {
 		if (is.null(cw)) cw <- 20
@@ -17,39 +17,6 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 	}
 		
 	
-	if (is.null(names(basemaps))) names(basemaps) <- sapply(basemaps, FUN = function(bm) {
-		if (substr(bm, 1, 4) == "http") {
-			x <- strsplit(bm, "/", fixed=TRUE)[[1]]
-			x <- x[-c(1, (length(x)-2):length(x))]
-			x <- x[x!=""]
-			paste(x, collapse="/")
-		} else bm
-	})
-	
-	if (!is.na(gt$set.zoom.limits[1])) {
-	  tileOptions <- lapply(basemaps.alpha, function(a) {
-	  	tileOptions(minZoom=gt$set.zoom.limits[1], maxZoom=gt$set.zoom.limits[2], opacity=a)
-	  })
-	  	
-	} else {
-	  tileOptions <- lapply(basemaps.alpha, function(a) {
-	  	tileOptions(opacity=a)
-	  })
-	}
-	
-	# add base layer(s)
-	if (length(basemaps)) {
-		for (i in 1:length(basemaps)) {
-			bm <- unname(basemaps[i])
-			bmname <- names(basemaps)[i]
-			if (substr(bm, 1, 4) == "http") {
-				lf <- lf %>% addTiles(bm, group=bmname, options=tileOptions[[i]])
-			} else {
-				lf <- lf %>% addProviderTiles(bm, group=bmname, options = tileOptions[[i]])
-			}
-		}
-	}
-	
 	# add background overlay
 	lf <- appendContent(lf, {
 		tags$head(
@@ -66,40 +33,125 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 	# 	}
 	# }
 	
-	if (!length(gp)) {
-		if (length(basemaps)>1) lf <- lf %>% addLayersControl(baseGroups=names(basemaps), options = layersControlOptions(autoZIndex = TRUE))
-
-		if (!is.null(gt$shape.bbx)) {
-			lf <- lf %>%
-				fitBounds(gt$shape.bbx[1], gt$shape.bbx[2], gt$shape.bbx[3], gt$shape.bbx[4]) %>%
-				addMarkers(gt$shape.center[1], gt$shape.center[2])
-		}
-		lf <- set_bounds_view(lf, gt)
-		return(lf)
-	}
+	# if (!length(gp)) {
+	# 	if (length(basemaps)>1) lf <- lf %>% addLayersControl(baseGroups=names(basemaps), options = layersControlOptions(autoZIndex = TRUE))
+	# 
+	# 	if (!is.null(gt$shape.bbx)) {
+	# 		lf <- lf %>%
+	# 			fitBounds(gt$shape.bbx[1], gt$shape.bbx[2], gt$shape.bbx[3], gt$shape.bbx[4]) %>%
+	# 			addMarkers(gt$shape.center[1], gt$shape.center[2])
+	# 	}
+	# 	lf <- set_bounds_view(lf, gt)
+	# 	return(lf)
+	# }
 	
 	e <- environment()
-	id <- 1
 	alpha <- gt$alpha
 
-	bbx <- attr(shps[[1]], "bbox")
+	gt$global_bbox_specified <- !is.null(gt$bbox)
 	
+	if (!gt$global_bbox_specified) {
+		gt$bbox <- attr(shps[[gt$shape.masterID]], "bbox")
+	}
+
 	warns <- c(symbol=FALSE, text=FALSE, raster=FALSE, symbol_legend=FALSE, linelwd_legend=FALSE) # to prevent a warning for each shape
 	
+	if (inherits(shps, "sf")) shps <- list(shps)
+
+	bases <- NA
+	overlays <- NA
+	overlays_tiles <- character(0)
+	
+	# should the layer control include base layers? TRUE if |basemaps| > 1 || names/groups are specified
+	basename.specified <- FALSE
+	
+	# workaround for https://github.com/rstudio/leaflet/issues/427
+	# pane <- "overlayPane"
+	# paneUsed <- FALSE
+	# nextPane <- function(pane) {
+	# 	pane <- "markerPane"  # when pane=overlayPane tiles are still below shadowPane
+	# 	# pane <- switch(pane,
+	# 	# 	  shadowPane = "overlayPane",
+	# 	# 	  "markerPane")
+	# 	assign("pane", pane, envir = e)
+	# 	assign("paneUsed", FALSE, envir = e)
+	# 	pane
+	# }
+	# usePane <- function() {
+	# 	assign("paneUsed", TRUE, envir = e)
+	# }
+	
+	
+	pane <- "overlayPane00"
+	paneID <- function(pane) as.integer(substr(pane, 12,14))
+	
+	nextPane <- function(pane) {
+		pane <- paste0("overlayPane", sprintf("%02d", paneID(pane) + 1))
+		assign("pane", pane, envir = e)
+		pane
+	} 
+	
+	
+	
+	addPane <- function(lf, pane) {
+		addMapPane(lf, pane, zIndex = paneID(pane) + 400)
+	}
+	
+
+	addBaseGroup <- function(group) {
+		for (g in group) {
+			if (is.na(bases[1])) {
+				bases <- g
+			} else if (!(g %in% bases)) {
+				bases <- c(bases, g)
+			}
+		}
+		assign("bases", bases, envir = e)
+	}
+	
+	eraseBaseGroup <- function() {
+		assign("bases", character(0), envir = e)
+	}
+	
+	eraseOverlayTiles <- function() {
+		overlays <- setdiff(overlays, overlays_tiles)
+		assign("overlays", overlays, envir = e)
+	}
+	
+	addOverlayGroup <- function(group, are.tiles = FALSE) {
+		if (is.na(overlays[1])) {
+			overlays <- group
+		} else if (!(group %in% overlays)) {
+			overlays <- c(overlays, group)
+		}
+		assign("overlays", overlays, envir = e)
+		if (are.tiles) assign("overlays_tiles", c(overlays_tiles, group), envir = e)
+	}
+	
 	group_selection <- mapply(function(shp, gpl, shp_name) {
-		bbx <- attr(shp, "bbox")
-		upl <- units_per_line(bbx)
-		bpl <- bbx_per_line(bbx)
-		if (inherits(shp, "Spatial")) {
-			res <- get_sp_coordinates(shp, gpl, gt, bbx)
-			co <- res$co
-			if (gt$shape.line.center.type[1]=="segment") {
-				gpl <- res$gpl
-				shp <- res$shp
-			}	
+		if (!is.null(shp)) {
+			if (nrow(shp) == 0) {
+				shp <- NULL
+			} else {
+				bbx <- attr(shp, "bbox")
+				upl <- units_per_line(bbx)
+				bpl <- bbx_per_line(bbx)
+				if (inherits(shp, "sf")) {
+					res <- get_sf_coordinates(shp, gpl)
+					co <- res$co
+					if (attr(shp, "point.per")=="segment") {
+						gpl <- res$gpl
+						shp <- res$shp
+					}
+					#co <- suppressWarnings(st_coordinates(st_geometry(st_centroid(shp))))
+				}
+			}
 		}
 		
+		
 		plot_tm_fill <- function() {
+			if (is.null(shp)) return(FALSE)
+			
 			bres <- split_alpha_channel(gpl$col, alpha=alpha)
 			bcol <- bres$col
 			bopacity <- bres$opacity
@@ -110,40 +162,69 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 			
 			if (!is.null(gpl$fill)) {
 				popups <- get_popups(gpl, type="fill")
+				labels <- get_labels(gpl, type="fill")
 			} else {
 				popups <- NULL
+				labels <- NULL
 			}
+
+			if (!is.null(labels)) shp$tmapID <- labels
+			
 			stroke <- gpl$lwd>0 && !is.na(bcol) && bopacity!=0
 			
 			charwidth <- attr(popups, "charwidth")
-	
-			lf <- lf %>% addPolygons(data=shp, stroke=stroke, weight=gpl$lwd, color=bcol, fillColor = fcol, opacity=bopacity, fillOpacity = fopacity, popup = popups, options = pathOptions(clickable=!is.null(popups)), group=shp_name, layerId = id, popupOptions = pOptions(charwidth))
 
+			group_name <- if (is.na(gpl$fill.group)) shp_name else gpl$fill.group
+			addOverlayGroup(group_name)
+			
+			pane <- nextPane(pane)
+			lf <- addPane(lf, pane)
+			
+			
+			lf <- lf %>% addPolygons(data=shp, label = ~tmapID, stroke=stroke, weight=gpl$lwd, color=bcol, fillColor = fcol, opacity=bopacity, fillOpacity = fopacity, popup = popups, options = pathOptions(clickable=!is.null(popups), pane=pane), group=group_name, popupOptions = pOptions(charwidth))
+			
+			# if (!is.null(labels)) {
+			# 	lf <- lf %>% 
+			# 		addSearchFeatures(targetGroups  = shp_name, options = searchFeaturesOptions(zoom = 7, openPopup=FALSE))
+			# }
 
 			if (!is.na(gpl$xfill[1])) {
-				if (gpl$fill.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="fill", alpha=alpha)
+				if (gpl$fill.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="fill", alpha=alpha, group = if (gt$free.scales.fill) group_name else NULL)
 			}
 
 			assign("lf", lf, envir = e)
-			assign("id", id+1, envir = e)
 			TRUE
 		}
 		
 		plot_tm_lines <- function() {
+			if (is.null(shp)) return(FALSE)
 			lres <- split_alpha_channel(gpl$line.col, alpha=alpha)
 			lcol <- lres$col
 			lopacity <- lres$opacity
 
 			popups <- get_popups(gpl, type="line")
+			labels <- get_labels(gpl, type="line")
 			charwidth <- attr(popups, "charwidth")
 			
 			dashArray <- lty2dashArray(gpl$line.lty)
 			
-			lf <- lf %>% addPolylines(data=shp, stroke=TRUE, weight=gpl$line.lwd, color=lcol, opacity = lopacity, popup = popups, options = pathOptions(clickable=!is.null(popups)), dashArray=dashArray, group=shp_name, layerId = id, popupOptions = pOptions(charwidth))
+			if (!is.null(labels)) shp$tmapID <- labels
+			
+			group_name <- if (is.na(gpl$line.group)) shp_name else gpl$line.group
+			addOverlayGroup(group_name)
+			
+			pane <- nextPane(pane)
+			lf <- addPane(lf, pane)
+			
+			lf <- lf %>% addPolylines(data=shp, label = ~tmapID, stroke=TRUE, weight=gpl$line.lwd, color=lcol, opacity = lopacity, popup = popups, options = pathOptions(clickable=!is.null(popups), pane=pane), dashArray=dashArray, group=group_name, popupOptions = pOptions(charwidth)) 
 
+			# if (!is.null(labels)) {
+			# 	lf <- lf %>% 
+			# 		addSearchFeatures(targetGroups  = shp_name, options = searchFeaturesOptions(zoom = 7, openPopup=FALSE))
+			# }
 			
 			if (!is.na(gpl$xline[1])) {
-				if (gpl$line.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="line.col", alpha=alpha)
+				if (gpl$line.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="line.col", alpha=alpha, group = if (gt$free.scales.line.lwd) group_name else NULL)
 			}
 			
 			if (!is.na(gpl$xlinelwd[1]) && gpl$line.lwd.legend.show) {
@@ -153,8 +234,7 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 			
 			
 			assign("lf", lf, envir = e)
-			assign("id", id+1, envir = e)
-			
+
 			TRUE
 			# 			col <- do.call("process_color", c(list(gpl$line.col, alpha=gpl$line.alpha), gt$pc))
 			# 			grid.shplines(shp, gp=gpar(col=col, lwd=gpl$line.lwd, lty=gpl$line.lty,
@@ -162,6 +242,7 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 		}
 		
 		plot_tm_symbols <- function() {
+			if (is.null(shp)) return(FALSE)
 			npol <- nrow(co)
 			
 			co[, 1] <- co[, 1] + gpl$symbol.xmod * bpl
@@ -199,6 +280,7 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 			}
 			
 			popups <- get_popups(gpl, type="symbol")
+			labels <- get_labels(gpl, type="symbol")
 			charwidth <- attr(popups, "charwidth")
 			
 			popups <- popups[sel]
@@ -212,12 +294,14 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 				
 				fcol2 <- if (length(fcol)==1) fcol else fcol[decreasing]
 				popups2 <- popups[decreasing]
+				labels2 <- labels[decreasing]
 			} else {
 				co2 <- co
 				symbol.size2 <- symbol.size
 				symbol.shape2 <- symbol.shape
 				fcol2 <- fcol
 				popups2 <- popups
+				labels2 <- labels
 			}
 			
 			
@@ -227,6 +311,13 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 			are.icons <- gpl$symbol.misc$symbol.are.icons
 			clustering <- gpl$symbol.misc$clustering
 			
+			group_name <- if (is.na(gpl$symbol.group)) shp_name else gpl$symbol.group
+			addOverlayGroup(group_name)
+			
+			pane <- nextPane(pane)
+			lf <- addPane(lf, pane)
+			
+			
 			if (are.icons) {
 				#symbol.size2 <- symbol.size2 / 3 # Correct for the fact that markers are larger than circle markers. This is good, but for static plots the icon size was already increased by icon.size=3, so this is to revert it for view mode
 				if (any(symbol.shape2<1000)) {
@@ -242,7 +333,7 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 						icons$iconAnchorY <- icons$iconAnchorY * symbol.size2
 					}
 				}
-				lf <- lf %>% addMarkers(lng = co2[,1], lat=co2[,2], popup=popups2, group=shp_name, icon=icons, layerId = id, clusterOptions=clustering)
+				lf <- lf %>% addMarkers(lng = co2[,1], lat=co2[,2], popup=popups2, label = labels2, group=group_name, icon=icons, clusterOptions=clustering, options = markerOptions(clickable=!is.null(popups), pane=pane))
 			} else {
 				if (!all(symbol.shape2 %in% c(1, 16, 19, 20, 21))) {
 					warns["symbol"] <- TRUE
@@ -250,16 +341,19 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 				}
 
 				if (fixed) {
-					lf <- lf %>% addCircleMarkers(lng=co2[,1], lat=co2[,2], fill = any(!is.na(fcol2)), fillColor = fcol2, fillOpacity=fopacity, color = bcol, stroke = !is.na(bcol) && bopacity!=0, radius = 20*symbol.size2, weight = 1, popup=popups2, group=shp_name, layerId = id, popupOptions = pOptions(charwidth), clusterOptions=clustering)
+					lf <- lf %>% addCircleMarkers(lng=co2[,1], lat=co2[,2], label = labels2, fill = any(!is.na(fcol2)), fillColor = fcol2, fillOpacity=fopacity, color = bcol, stroke = !is.na(bcol) && bopacity!=0, radius = 20*symbol.size2, weight = 1, popup=popups2, group=group_name, popupOptions = pOptions(charwidth), clusterOptions=clustering, options = pathOptions(clickable=!is.null(popups), pane=pane))
 				} else {
-					lf <- lf %>% addCircles(lng=co2[,1], lat=co2[,2], fill = any(!is.na(fcol2)), fillColor = fcol2, fillOpacity=fopacity, color = bcol, stroke = !is.na(bcol) && bopacity!=0, radius=rad, weight =1, popup=popups2, group=shp_name, layerId = id, popupOptions = pOptions(charwidth))
+					lf <- lf %>% addCircles(lng=co2[,1], lat=co2[,2], label = labels2, fill = any(!is.na(fcol2)), fillColor = fcol2, fillOpacity=fopacity, color = bcol, stroke = !is.na(bcol) && bopacity!=0, radius=rad, weight =1, popup=popups2, group=group_name, popupOptions = pOptions(charwidth), options = pathOptions(clickable=!is.null(popups), pane=pane))
 				}
 			}
 				
-			
+			# if (!is.null(labels)) {
+			# 	lf <- lf %>% 
+			# 		addSearchFeatures(targetGroups  = shp_name, options = searchFeaturesOptions(zoom = 7, openPopup=FALSE))
+			# }
 			
 			if (!is.na(gpl$xcol[1])) {
-				if (gpl$symbol.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="symbol.col", alpha=alpha)
+				if (gpl$symbol.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="symbol.col", alpha=alpha, group = if (gt$free.scales.symbol.col) group_name else NULL)
 			}
 			
 			if (!is.na(gpl$xsize[1]) && gpl$symbol.size.legend.show) {
@@ -270,11 +364,11 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 			
 
 			assign("lf", lf, envir = e)
-			assign("id", id+1, envir = e)
 			TRUE
 			
 		}
 		plot_tm_text <- function() {
+			if (is.null(shp)) return(FALSE)
 # 			if (is.null(gpl$symbol.misc) || !gpl$symbol.misc$symbol.are.markers) {
 # 				warns["text"] <- TRUE
 # 				assign("warns", warns, envir = e)
@@ -304,98 +398,210 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 			sizeChar <- paste(round(size * 12), "px", sep="")
 			colsize <- paste(col, sizeChar, sep="_^_")
 			
-			just <- switch(gpl$text.just[1],
-						   left="right", 
-						   right="left",
-						   "bottom")
+			direction <- ifelse(gpl$text.just == "left", "right",
+						 ifelse(gpl$text.just == "right", "left",
+						 ifelse(gpl$text.just == "top", "bottom",
+						 ifelse(gpl$text.just == "bottom", "top", "center"))))
 			
 				
 			cs_set <- unique(colsize)
 			
+			clustering <- gpl$text.misc$clustering
+			
+			group_name <- if (is.na(gpl$text.group)) shp_name else gpl$text.group
+			addOverlayGroup(group_name)
+			
+			pane <- nextPane(pane)
+			lf <- addPane(lf, pane)
+			
+			
 			if (length(cs_set)==1) {
 				lf <- lf %>% addLabelOnlyMarkers(lng = co[,1], lat = co[,2], label=text,
-												 labelOptions = labelOptions(noHide = TRUE, textOnly = TRUE, direction = just, 
+												 group=group_name, 
+												 labelOptions = labelOptions(noHide = TRUE, textOnly = TRUE, direction = direction, 
 												 							opacity=opacity,
 												 							textsize=sizeChar[1],
-												 							style=list(color=col[1])))
+												 							style=list(color=col[1])),
+												 clusterOptions = clustering,
+												 options = markerOptions(pane = pane))
 			} else {
 				for (i in 1:length(text)) {
 					lf <- lf %>% addLabelOnlyMarkers(lng = co[i,1], lat = co[i,2], label=text[i],
-													 labelOptions = labelOptions(noHide = TRUE, textOnly = TRUE, direction = just, 
+													 group=group_name, 
+													 labelOptions = labelOptions(noHide = TRUE, textOnly = TRUE, direction = direction, 
 													 							opacity=opacity,
 													 							textsize=sizeChar[i],
-													 							style=list(color=col[i])))	
+													 							style=list(color=col[i])),
+													 clusterOptions = clustering,
+													 options = markerOptions(pane = pane))	
 				}
 			}
 			
 			if (!is.na(gpl$xtcol[1])) {
-				if (gpl$text.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="text.col", alpha=alpha)
+				if (gpl$text.col.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="text.col", alpha=alpha, group = if (gt$free.scales.text.col) group_name else NULL)
 			}
 			
 			assign("lf", lf, envir = e)
-			assign("id", id+1, envir = e)
-			
+
 			TRUE
 		}
 		plot_tm_raster <- function() {
+			if (is.null(shp)) return(FALSE)
 			if (gpl$raster.misc$is.OSM) {
-				if (is.na(gpl$raster.misc$leaflet.provider)) {
+				if (is.na(gpl$raster.misc$leaflet.server)) {
 					warns["raster"] <- TRUE
 					assign("warns", warns, envir = e)
 				} else {
-					if (gpl$raster.misc$leaflet.provider==gt$basemaps[1]) {
-						warns["raster"] <- gpl$raster.misc$leaflet.provider
+					if (gpl$raster.misc$leaflet.server==gt$basemaps[1]) {
+						warns["raster"] <- gpl$raster.misc$leaflet.server
 						assign("warns", warns, envir = e)
 					}
 				}
 				return(FALSE)	
 			}
 			if (is.na(gpl$xraster[1])) {
-				gpl$raster.legend.palette <- na.omit(unique(gpl$raster))
 				gpl$raster.legend.values <- 1
 			}
 			
 			shp@data@values <- match(gpl$raster, gpl$raster.legend.palette)
 
-			res_leg <- add_legend(map=NULL, gpl, gt, aes="raster", alpha = alpha, list.only=TRUE)
 			
-			lf <- lf %>% addRasterImage(x=shp, colors=res_leg$col, opacity = res_leg$opacity, group=shp_name, project = FALSE, layerId = id)
+			group_name <- if (is.na(gpl$raster.group)) shp_name else gpl$raster.group
+			addOverlayGroup(group_name)
+			
+			res_leg <- add_legend(map=NULL, gpl, gt, aes="raster", alpha = alpha, group = if (gt$free.scales.raster) group_name else NULL, list.only=TRUE)
+			
+			lf <- lf %>% addRasterImage(x=shp, colors=res_leg$col, opacity = res_leg$opacity, group=group_name, project = FALSE)
 			
 			if (!is.na(gpl$xraster[1])) {
-				if (gpl$raster.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="raster", alpha=alpha)
+				if (gpl$raster.legend.show) lf <- lf %>% add_legend(gpl, gt, aes="raster", alpha=alpha, group = if (gt$free.scales.raster) group_name else NULL)
 			}
 
 			assign("lf", lf, envir = e)
-			assign("id", id+1, envir = e)
 			TRUE
 		}
 		plot_tm_grid <- function() {
 			lf <- lf %>% addGraticule()
 			
 			assign("lf", lf, envir = e)
-			assign("id", id+1, envir = e)
+			TRUE
+		}
+		
+		plot_tm_tiles <- function() {
+			basemaps <- gpl$tile.server
+			basemaps.alpha <- gpl$tile.alpha
+			type <- gpl$tile.grouptype
+			
+			if (is.null(basemaps)) {
+				return(FALSE)
+			}
+			
+			if (is.na(basemaps[1])) {
+				if (type == "base") eraseBaseGroup() else eraseOverlayTiles()
+				return(FALSE) 
+			}
+			
+
+			
+			
+			group_names <- if (is.na(gpl$tile.group[1])) {
+				vapply(basemaps, FUN = function(bm) {
+					if (substr(bm, 1, 4) == "http") {
+						x <- strsplit(bm, "/", fixed=TRUE)[[1]]
+						x <- x[-c(1, (length(x)-2):length(x))]
+						x <- x[x!=""]
+						paste(x, collapse="/")
+					} else bm
+				}, character(1))
+			} else {
+				if (type == "base") assign("basename.specified", TRUE, envir = e)
+				rep(gpl$tile.group, length.out = length(basemaps))
+			}
+			
+			
+			if (type == "base") {
+				addBaseGroup(group_names)	
+			} else {
+				addOverlayGroup(group_names, are.tiles = TRUE)
+			}
+			
+			if(type == "base") {
+				pane <- "tilePane"
+			} else {
+				pane <- nextPane(pane)
+				lf <- addPane(lf, pane)
+			}
+				
+			if (!is.na(gt$set.zoom.limits[1])) {
+				tileOptions <- lapply(basemaps.alpha, function(a) {
+					tileOptions(minZoom=gt$set.zoom.limits[1], maxZoom=gt$set.zoom.limits[2], opacity=a, pane=pane)
+				})
+				
+			} else {
+				tileOptions <- lapply(basemaps.alpha, function(a) {
+					tileOptions(opacity=a, pane=pane)
+				})
+			}
+			
+			# add base layer(s)
+			if (length(basemaps)) {
+				for (i in 1:length(basemaps)) {
+					bm <- unname(basemaps[i])
+					bmname <- unname(group_names[i])
+					if (substr(bm, 1, 4) == "http") {
+						lf <- lf %>% addTiles(bm, group=bmname, options=tileOptions[[i]])
+					} else {
+						lf <- lf %>% addProviderTiles(bm, group=bmname, options = tileOptions[[i]])
+					}
+				}
+			}
+			assign("lf", lf, envir = e)
+
 			TRUE
 		}
 		
 		e2 <- environment()
 		
 		fnames <- paste("plot", gpl$plot.order, sep="_")
-		layer_selection <- sapply(fnames, do.call, args=list(), envir=e2)
+		layer_selection <- vapply(fnames, do.call, logical(1), args=list(), envir=e2)
 		any(layer_selection)
 	}, shps, gp, gt$shp_name, SIMPLIFY = TRUE)
 	
-	if (showWarns) {
+	if (gt$show.messages && showWarns) {
 		if (warns["symbol"]) message("Symbol shapes other than circles or icons are not supported in view mode.")
 		if (warns["symbol_legend"]) message("Legend for symbol sizes not available in view mode.")
 		if (warns["linelwd_legend"]) message("Legend for line widths not available in view mode.")
 		if (identical(unname(warns["raster"]), TRUE)) {
-			message("Raster data contains OpenStreetMapData (read with read_osm) from provider that is not known from http://leaflet-extras.github.io/leaflet-providers/preview/index.html")	
+			message("OpenStreetMapData read with read_osm is static, so not usable in view mode. Please use tm_basemap or tm_tiles.")	
 		} else if (!(identical(unname(warns["raster"]), FALSE))) {
-			message("Raster data contains OpenStreetMapData (read with read_osm). Therefore, the basemap has been set to \"", warns["raster"], "\"")
+			message("OpenStreetMapData read with read_osm is static, so not usable in view mode. Please use tm_basemap or tm_tiles, with the provider name set to \"", warns["raster"], "\"")
 		}
 	}
 	
-	groups <- gt$shp_name[group_selection]
+	## add manual legends (tm_add_legend)
+	if (length(gal) > 0) {
+		for (gali in gal) {
+			if (gali$type != "fill") {
+				if (gt$show.messages) {
+					message("only legends of type \"fill\" supported in view mode")
+				}
+			} else {
+				RGBA <- col2rgb(gali$col, alpha = TRUE)
+				col <- rgb(RGBA[1,], RGBA[2,], RGBA[3,], maxColorValue = 255)
+				opacity <- unname(RGBA[4,1]/255) * alpha
+				
+				lf <- lf %>% addLegend(position=gt$view.legend.position,
+									   colors = col,
+									   labels = gali$labels,
+									   title=gali$title, 
+									   opacity=opacity)
+			}
+		}
+	}
+	
+	
+	
+	#groups <- gt$shp_name[group_selection]
 	
 	#center <- c(mean(lims[c(1,3)]), mean(lims[c(2,4)]))
 	
@@ -410,14 +616,60 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 		stop("Invalid control.position", call.=FALSE)
 	}
 
+	if (length(bases) == 1 && !basename.specified) {
+		if (!is.na(overlays[1])) {
+			lf <- lf %>% addLayersControl(overlayGroups = unname(overlays), options = layersControlOptions(autoZIndex = TRUE), position=control.position)
+		}
+	} else if (!is.na(overlays[1])) {
+		lf <- lf %>% addLayersControl(baseGroups=unname(bases), overlayGroups = unname(overlays), options = layersControlOptions(autoZIndex = TRUE), position=control.position)  
+	} else {
+		lf <- lf %>% addLayersControl(baseGroups=unname(bases), options = layersControlOptions(autoZIndex = TRUE), position=control.position)  
+	}
 	
-	lf <- lf %>% addLayersControl(baseGroups=names(basemaps), overlayGroups = groups, options = layersControlOptions(autoZIndex = TRUE), position=control.position)  
 
 	if (gt$scale.show) {
-		u <- gt$shape.units_args$target
+		u <- gt$shape.units$unit
 		metric <- (u %in% c("m", "km", "metric"))
 	 	lf <- lf %>% addScaleBar(position = gt$scale.position, options = scaleBarOptions(maxWidth=gt$scale.width, metric=metric, imperial = !metric))
 	}
+	
+	if (gt$minimap.show) {
+		mmargs <- gt[substr(names(gt), 1, 4) == "mini"]
+		names(mmargs) <- substr(names(mmargs), 9, nchar(names(mmargs)))
+		names(mmargs)[names(mmargs) == "toggle"] <- "toggleDisplay"
+		names(mmargs)[names(mmargs) == "server"] <- "tiles"
+		mmargs$show <- NULL
+		
+		specified_tiles <- !is.na(mmargs$tiles)
+		
+		if (!specified_tiles) {
+			if (length(bases) == 0) {
+				mmargs$tiles <- NULL
+			} else {
+				mmargs$tiles <- bases[1]	
+			} 	
+		} 
+		
+		lf <- tryCatch({
+			lf <- do.call(addMiniMap, c(list(map = lf), mmargs)) 
+			if (!specified_tiles && (length(bases) > 0)) {
+				lf <- lf %>% 
+					htmlwidgets::onRender("
+			    function(el, x) {
+			      var myMap = this;
+			      myMap.on('baselayerchange',
+			        function (e) {
+			          myMap.minimap.changeLayer(L.tileLayer.provider(e.name));
+			        })
+			    }")
+			}
+			lf
+		}, error = function(e) {
+			if (gt$show.messages) message("tm_minimap requires the leaflet package. Please run library(leaflet)")
+			lf
+		})
+	}
+	
 	# print(leaflet_id)
 	# if (gt$title!="") {
 	# 	lf <- lf %>% onRender(paste("
@@ -430,13 +682,21 @@ view_tmap <- function(gp, shps=NULL, leaflet_id=1, showWarns=TRUE) {
 	# 		}", sep=""))
 	# }
 	
-	lf <- set_bounds_view(lf, gt, bbx)
+	
+
+	lf <- set_bounds_view(lf, gt)
 	lf$title <- gt$title
 	lf
 }
 
-set_bounds_view <- function(lf, gt, bbx) {
-	if (is.logical(gt$set.bounds)) {
+set_bounds_view <- function(lf, gt) {
+	# if (!is.null(gt$shape.bbx)) {
+	# 	lf <- lf %>%
+	# 		fitBounds(gt$shape.bbx[1], gt$shape.bbx[2], gt$shape.bbx[3], gt$shape.bbx[4]) %>%
+	# 		addMarkers(gt$shape.center[1], gt$shape.center[2])
+	# }
+	
+	if (is.logical(gt$set.bounds) && !is.null(lf$x$limits)) {
 		lims <- unname(unlist(lf$x$limits)[c(3,1,4,2)])
 	} else {
 		lims <- gt$set.bounds
@@ -449,18 +709,31 @@ set_bounds_view <- function(lf, gt, bbx) {
 			gt$set.view <- c(mean(lims[c(1,3)]), mean(lims[c(2,4)]), gt$set.zoom.limits[1])
 		}
 	}
-	
-	if (!is.na(gt$set.view[1])) {
-		lf <- lf %>% setView(gt$set.view[1], gt$set.view[2], gt$set.view[3])
-	} else if (!missing(bbx)) {
-		bbx <- unname(bbx)
-		lf <- lf %>% fitBounds(bbx[1], bbx[2], bbx[3], bbx[4]) #setView(view[1], view[2], view[3])
+	if (length(gt$set.view) == 1 && !is.na(gt$set.view[1])) {
+		gt$set.view <- c(mean(lims[c(1,3)]), mean(lims[c(2,4)]), gt$set.view)
 	}
+	
+	if (!is.na(gt$set.view[1]) && !gt$global_bbox_specified) {
+		set.view <- gt$set.view
+		
+		if (!is.null(names(set.view))) {
+			if (!setequal(names(set.view), c("lon", "lat", "zoom"))) stop("Incorrect set.view names. They should be \"lon\", \"lat\", and \"zoom\"", call. = FALSE)
+			set.view <- unname(set.view[c("lon", "lat", "zoom")])
+		}
+		
+		lf <- lf %>% setView(set.view[1], set.view[2], set.view[3])
+	} else {
+		bbx <- unname(gt$bbox)
+		if (!is.null(bbx)) lf <- lf %>% fitBounds(bbx[1], bbx[2], bbx[3], bbx[4]) #setView(view[1], view[2], view[3])
+	}
+	
+	if (!is.null(gt$center)) lf <- lf %>% addMarkers(gt$center[1], gt$center[2])
+	
 	lf
 }
 
 format_popups <- function(id=NULL, titles, format, values) {
-	isnull <- sapply(values, is.null)
+	isnull <- vapply(values, is.null, logical(1))
 	
 	titles <- titles[!isnull]
 	titles[names(titles)!=""] <- names(titles)[names(titles)!=""]
@@ -479,9 +752,15 @@ format_popups <- function(id=NULL, titles, format, values) {
 		labels <- ""
 	}
 	
-	titles_format <- sapply(titles, htmlEscape)
+	titles_format <- vapply(titles, htmlEscape, character(1))
 	values_format <- mapply(function(v, f) {
-		htmlEscape(if (is.numeric(v)) do.call("fancy_breaks", c(list(vec=v, intervals=FALSE), f)) else v)
+		if (inherits(v, "units")) {
+			popup_append <- paste0(" ", as.character(attr(v, "units")))
+		} else {
+			popup_append <- ""
+		}
+		numbers <- htmlEscape(if (is.numeric(v)) do.call("fancy_breaks", c(list(vec=as.numeric(v), intervals=FALSE), f)) else v)
+		paste0(numbers, popup_append)
 	}, values, format, SIMPLIFY = FALSE)
 	
 	
@@ -536,6 +815,11 @@ get_aes_name <- function(type) {
 	}
 }
 
+get_labels <- function(gpl, type) {
+	var_names <- paste(type, "names", sep=".")
+	gpl$data[[gpl[[var_names]]]]
+}
+
 get_popups <- function(gpl, type) {
 	var_names <- paste(type, "names", sep=".")
 	var_vars <- paste(type, "popup.vars", sep=".")
@@ -551,7 +835,7 @@ get_popups <- function(gpl, type) {
 	popups
 }
 
-add_legend <- function(map, gpl, gt, aes, alpha, list.only=FALSE) {
+add_legend <- function(map, gpl, gt, aes, alpha, group, list.only=FALSE) {
 	pal_name <- paste(aes, "legend.palette", sep=".")
 	val_name <- paste(aes, "legend.values", sep=".")
 	lab_name <- paste(aes, "legend.labels", sep=".")
@@ -561,13 +845,14 @@ add_legend <- function(map, gpl, gt, aes, alpha, list.only=FALSE) {
 	lab <- gpl[[lab_name]]
 	
 	if (nchar(pal[1])>10) {
+		# check whether style is continuous
 		style <- attr(pal, "style")
 		is.cont <- TRUE
 		incl.na <- nchar(pal[length(pal)]) < 10
-		pal <- sapply(pal, function(x) {
+		pal <- vapply(pal, function(x) {
 			p <- strsplit(x, split = "-", fixed=TRUE)[[1]]
 			if (length(p)==1) p[1] else if (p[6]=="NA") p[5] else p[6]
-		})
+		}, character(1))
 		if (incl.na) {
 			colNA <- unname(pal[length(pal)])
 			pal <- pal[-length(pal)]
@@ -620,19 +905,19 @@ add_legend <- function(map, gpl, gt, aes, alpha, list.only=FALSE) {
 		legvals <- if (!is.na(colNA)) c(val, NA) else val
 
 		if (style=="quantile") {
-			addLegend(map, position=legend.position,
+			addLegend(map, position=legend.position, group = group,
 					  pal=colorQuantile(col, val, na.color=colNA, alpha = FALSE), values=legvals, na.label = textNA, title=title, opacity=opacity)
 		} else {
-			addLegend(map, position=legend.position,
+			addLegend(map, position=legend.position, group = group,
 					  pal=colorNumeric(col, val, na.color=colNA, alpha = FALSE), values=legvals, na.label = textNA, title=title, opacity=opacity)
 		}
 	} else {
 		if (allNAs) {
-			addLegend(map, position=legend.position, colors=colNA, labels=textNA, title=title, opacity=opacity)
+			addLegend(map, position=legend.position, group = group, colors=colNA, labels=textNA, title=title, opacity=opacity)
 		} else {
 			legvals <- if (!is.na(colNA)) factor(c(lab, NA), levels=lab) else factor(lab, levels=lab)
 			lab <- factor(lab, levels=lab)
-			addLegend(map, position=legend.position, 
+			addLegend(map, position=legend.position, group = group,
 					  pal=colorFactor(col, domain=lab, na.color = colNA, ordered = TRUE, alpha = FALSE), values = legvals, na.label=textNA, title=title, opacity=opacity)
 		}
 	}
@@ -644,14 +929,13 @@ working_internet <- function(url = "http://www.google.com") {
 	
 	# test connection by trying to read first line of url
 	test <- try(suppressWarnings(readLines(url, n = 1)), silent = TRUE)
-	
 	# return FALSE if test inherits 'try-error' class
 	!inherits(test, "try-error")
 }
 
 bbx_per_line <- function(bbx) {
 	max_lines <- 60 #par("din")[2]*10 #disabled because window height doesn't influence scaling
-	(bbx[2,2] - bbx[2,1]) / max_lines
+	(bbx[4] - bbx[2]) / max_lines
 }
 
 units_per_line <- function(bbx) {
@@ -680,10 +964,16 @@ lty2dashArray <- function(lty) {
 }
 
 get_epsg_number <- function(proj) {
+	if (inherits(proj, "crs")) {
+		if (!is.na(proj$epsg)) {
+			return(proj$epsg)	
+		} else {
+			proj <- proj$proj4string
+		}
+	}
 	if (inherits(proj, "CRS")) proj <- attr(proj, "projargs")
 	
 	pat <- "^.*\\=epsg ?: ?(\\S*)(.*)$"
 	epsg <- as.numeric(sub(pat, "\\1", proj[grepl(pat, proj)]))
 	if (length(epsg)==0) NA else epsg
 }
-
